<?php

/**
 * @table paysystems
 * @id recurly
 * @title Recurly
 * @visible_link http://www.recurly.com/
 * @am_payment_api 6.0
 */
class Am_Paysystem_Recurly extends Am_Paysystem_CreditCard
{
    
    const
        PLUGIN_STATUS = self::STATUS_DEV,
        PLUGIN_DATE = '$Date$',
        PLUGIN_REVISION = '6.3.21',
        URL_RECURLY_JS = 'https://js.recurly.com/v4/recurly.js',
        URL_RECURLY_CSS = 'https://js.recurly.com/v4/recurly.css',
        TOKEN = 'recurly-account-id',
        CC_EXPIRES = 'recurly-cc-expires',
        CC_MASKED = 'recurly-cc-masked',
        API_VERSION = '2.10',
        API_ENDPOINT_TMPL = 'https://%s.recurly.com/v2/%s';
    
    protected
        $defaultTitle = "Recurly - pay with your Credit Card",
        $defaultDescription = "accepts all major credit cards",
        $_pciDssNotRequired = true;
    
    function getApiEndpoint($func)
    {
        return sprintf(self::API_ENDPOINT_TMPL, $this->getConfig('subdomain'), $func);
    }
    
    function init()
    {
        $this->getDi()->billingPlanTable->customFields()->add(new Am_CustomFieldText('recurly_plan_code',
            ___('Recurly Plan Code')));
    }
    
    public
    function storesCcInfo()
    {
        return false;
    }
    
    public
    function getRecurringType()
    {
        return self::REPORTS_CRONREBILL;
    }
    
    public
    function allowPartialRefunds()
    {
        return true;
    }
    
    public
    function getSupportedCurrencies()
    {
        return array_keys(Am_Currency::getFullList());
    }
    
    public
    function _initSetupForm(
        Am_Form_Setup $form
    ) {
        $form->addText("subdomain")
            ->setLabel(___("Your Recurly Account\n"
                . "You can get it from subdomain that you have set during account registration\n"
                . "For example if you have subdomain set: example.recurly.com\n"
                . "Your account is 'example'"))
            ->addRule('required');
        
        $form->addSecretText("api_key", ['size' => 40])
            ->setLabel("Recurly Private API Key\n" .
                "from Recurly Account->Developers->API Credentials")
            ->addRule('required');
        
        $form->addText("api_public_key", ['size' => 40])
            ->setLabel("Recurly Public API Key\n" .
                "from Recurly Account->Developers->API Credentials")
            ->addRule('required');
    }
    
    public
    function isConfigured()
    {
        return (bool)($this->getConfig('subdomain') && $this->getConfig('api_key'));
    }
    
    function storeCreditCard(CcRecord $cc, Am_Paysystem_Result $result)
    {
    
    }
    
    public
    function loadCreditCard(
        Invoice $invoice
    ) {
        if ($invoice->getUser()->data()->get(self::TOKEN)) {
            return $this->getDi()->CcRecordTable->createRecord();
        } // return fake record for rebill
    }
    
    function createController(Am_Mvc_Request $request, Am_Mvc_Response $response, array $invokeArgs)
    {
        return new Am_Mvc_Controller_CreditCard_Recurly($request, $response, $invokeArgs);
    }
    
    public
    function _doBill(
        Invoice $invoice,
        $doFirst,
        CcRecord $cc,
        Am_Paysystem_Result $result
    ) {
        if (!$token = $invoice->getUser()->data()->get(self::TOKEN)) {
            return $result->setErrorMessages([___('Payment failed')]);
        }
        if ($doFirst && (doubleval($invoice->first_total) <= 0)) { // free trial
            $tr = new Am_Paysystem_Transaction_Free($this);
            $tr->setInvoice($invoice);
            $tr->process();
            $result->setSuccess($tr);
        } else {
            $tr = new Am_Paysystem_Transaction_Recurly_Transaction($this, $invoice, $doFirst);
            $tr->run($result);
        }
    }
    
    public
    function getUpdateCcLink(
        $user
    ) {
        if (!$user->data()->get(self::TOKEN)) {
            return;
        }
        if (!$this->getDi()->invoiceTable->findFirstBy([
            'user_id' => $user->pk(),
            'paysys_id' => $this->getId(),
            'status' => Invoice::RECURRING_ACTIVE
        ])) {
            
            return;
        }
        return $this->getPluginUrl('update');
    }
    
    public
    function processRefund(
        InvoicePayment $payment,
        Am_Paysystem_Result $result,
        $amount
    ) {
        $tr = new Am_Paysystem_Transaction_Recurly_Refund($this, $payment->getInvoice(), $payment->receipt_id, $amount);
        $tr->run($result);
    }
    
    public function getReadme()
    {
        $ipn = $this->getPluginUrl('ipn');
        return <<<CUT
        Recurly Payment Plugin Readme

Set Webhook URL to - $ipn

CUT;
    }
    
    function canAutoCreate()
    {
        return true;
    }
    
    function createTransaction($request, $response, array $invokeArgs)
    {
        return new Am_Paysystem_Transaction_Recurly_Incoming($this, $request, $response, $invokeArgs);
    }
    
}

class Am_Mvc_Controller_CreditCard_Recurly extends Am_Mvc_Controller
{
    
    protected
        /** @var Invoice */
        $invoice,
        /** @var Am_Paysystem_Recurly */
        $plugin;
    
    function setInvoice(Invoice $invoice)
    {
        $this->invoice = $invoice;
    }
    
    function setPlugin($plugin)
    {
        $this->plugin = $plugin;
    }
    
    function createForm($label, $cc_mask = null)
    {
        $form = new Am_Form('payment-form');
        $name = $form->addGroup()
            ->setLabel(___("Cardholder Name\n" .
                'cardholder first and last name, exactly as on the card'));
        $name->addRule('required', ___('Please enter credit card holder name'));
        $name->setSeparator(' ');
        $name_f = $name->addText('cc_name_f', ['size' => 15, 'id' => 'cc_name_f', 'data-recurly' => 'first_name']);
        $name_f->addRule('required', ___('Please enter credit card holder first name'))->addRule('regex',
            ___('Please enter credit card holder first name'), '|^[a-zA-Z_\' -]+$|');
        $name_l = $name->addText('cc_name_l', ['size' => 15, 'id' => 'cc_name_l', 'data-recurly' => 'last_name']);
        $name_l->addRule('required', ___('Please enter credit card holder last name'))->addRule('regex',
            ___('Please enter credit card holder last name'), '|^[a-zA-Z_\' -]+$|');
        
        
        $form->addStatic('recurly-cc-info')
            ->setContent(<<<EOT
                <div data-recurly="card" id='cc_number'></div>
EOT
            )->setLabel(___('Credit Card Info'));
        
        $fieldSet = $form->addFieldset(___('Address Info'))
            ->setLabel(___("Address Info\n" .
                '(must match your credit card statement delivery address)'));
        $street = $fieldSet->addText('cc_street', ['data-recurly' => 'address1'])->setLabel(___('Street Address'))
            ->addRule('required', ___('Please enter Street Address'));
        
        $city = $fieldSet->addText('cc_city', ['data-recurly' => 'city'])->setLabel(___('City '))
            ->addRule('required', ___('Please enter City'));
        
        $zip = $fieldSet->addText('cc_zip', ['data-recurly' => 'postal_code'])->setLabel(___('ZIP'))
            ->addRule('required', ___('Please enter ZIP code'));
        
        $country = $fieldSet->addSelect('cc_country', ['data-recurly' => 'country'])->setLabel(___('Country'))
            ->setId('f_cc_country')
            ->loadOptions(Am_Di::getInstance()->countryTable->getOptions(true));
        $country->addRule('required', ___('Please enter Country'));
        
        $group = $fieldSet->addGroup()->setLabel(___('State'));
        $group->addRule('required', ___('Please enter State'));
        /** @todo load correct states */
        $stateSelect = $group->addSelect('cc_state', ['data-recurly' => 'state'])
            ->setId('f_cc_state')
            ->loadOptions($stateOptions = Am_Di::getInstance()->stateTable->getOptions(@$_REQUEST['cc_country'], true));
        $stateText = $group->addText('cc_state')->setId('t_cc_state');
        $disableObj = $stateOptions ? $stateText : $stateSelect;
        $disableObj->setAttribute('disabled', 'disabled')->setAttribute('style', 'display: none');
        
        $form->addSubmit('', ['value' => $label, 'id' => 'am-pay-form-submit']);
        
        $form->addHidden('id')->setValue($this->_request->get('id'));
        $form->addHidden('recurly-token', ['data-recurly' => 'token']);
        
        $form->setDataSources([$this->_request]);
        $form->addScript()->setScript(<<<RECJS
jQuery(document).ready(function(){
   jQuery('#payment-form').on('submit', function (event) {
        var form = this;
        
        event.preventDefault();
        jQuery(form).find("input[type=submit]").prop('disabled', 'disabled');
        recurly.token(form, function (err, token) {
        if (err) {
            jQuery(form).find("input[type=submit]").prop('disabled', null);
            var el = jQuery(jQuery.find("#cc_number"));
            msg = err.message;
            if(err.fields)
                msg = msg + " : "+err.fields.join(', ');
            var cnt = el.closest(".am-element");
            
            cnt.addClass("am-error");
            cnt.find("span.am-error").remove();
            el.after("<span class='am-error'><br />"+msg+"</span>");
        } else {
          // recurly.js has filled in the 'token' field, so now we can submit the
          // form to your server; alternatively, you can access token.id and do
          // any processing you wish
          form.submit();
        }
    });
    
  });
});
RECJS
        );
        return $form;
    }
    
    public
    function updateAction()
    {
        $user = $this->getDi()->user;
        if (!$token = $user->data()->get(Am_Paysystem_Recurly::TOKEN)) {
            throw new Am_Exception_Paysystem("No credit card stored, nothing to update");
        }
        if (!$this->invoice = $this->getDi()->invoiceTable->findFirstBy(
            ['user_id' => $user->pk(), 'paysys_id' => $this->plugin->getId()], 'invoice_id DESC')) {
            
            throw new Am_Exception_Paysystem("No invoices found for user and paysystem");
        }
        
        $tr = new Am_Paysystem_Transaction_Recurly_FindAccount($this->plugin, $this->invoice, $token);
        $tr->run(new Am_Paysystem_Result());
        if (!$tr->getUniqId()) // cannot load profile
        { // todo delete old profile, and display cc form again!
            throw new Am_Exception_Paysystem("Could not load customer profile");
        }
        
        $this->form = $this->createForm(___('Update Credit Card Info'));
        $result = $this->ccFormAndSaveCustomer();
        
        if ($result->isSuccess()) {
            $this->_redirect($this->getDi()->surl('member', false));
        }
        $public_key = json_encode($this->plugin->getConfig('api_public_key'));
        $this->view->headScript()->appendScript(<<<INITJS
jQuery(document).ready(function(){
    recurly.configure({$public_key});
});
INITJS
        );
        
        $this->view->headScript()->appendFile(Am_Paysystem_Recurly::URL_RECURLY_JS);
        $this->view->headLink()->appendStylesheet(Am_Paysystem_Recurly::URL_RECURLY_CSS);
        $this->view->headStyle()->appendStyle(<<<INITSTYLE
INITSTYLE
        );
        
        $this->view->title = ___('Payment info');
        $this->view->display_receipt = false;
        $this->view->form = $this->form;
        $this->view->display('cc/info.phtml');
    }
    
    protected
    function ccFormAndSaveCustomer()
    {
        $vars = $this->form->getValue();
        $result = new Am_Paysystem_Result();
        if ($token = $this->_request->get('recurly-token')) {
            $custFindTr = new Am_Paysystem_Transaction_Recurly_FindAccount($this->plugin, $this->invoice);
            $custFindTr->run($result);
            if ($result->isFailure()) {
                $tr = new Am_Paysystem_Transaction_Recurly_CreateCustomer($this->plugin, $this->invoice, $token);
            } else {
                $tr = new Am_Paysystem_Transaction_Recurly_UpdateCustomer($this->plugin, $this->invoice, $token);
            }
            $result->reset();
            $tr->run($result);
            if ($result->isSuccess()) {
                $this->invoice->getUser()->data()
                    ->set(Am_Paysystem_Recurly::TOKEN, $tr->getUniqId());
                $billingInfoTr = new Am_Paysystem_Transaction_Recurly_GetBillingInfo($this->plugin, $this->invoice);
                $billingInfoTr->run(new Am_Paysystem_Result());
                
                if ($billingInfo = $billingInfoTr->getBillingInfo()) {
                    $this->invoice->getUser()->data()
                        ->set(Am_Paysystem_Recurly::CC_EXPIRES,
                            sprintf('%02d%02d', (string)$billingInfo->month, (int)$billingInfo->year - 2000))
                        ->set(Am_Paysystem_Recurly::CC_MASKED, 'XXXX' . (string)$billingInfo->last_four
                        );
                    // setup session to do not reask payment info within 30 minutes
                    $s = $this->getDi()->session->ns($this->plugin->getId());
                    $s->setExpirationSeconds(60 * 30); // after 30 minutes we will reset the session
                    $s->ccConfirmed = true;
                }
                $this->invoice->getUser()->data()->update();
            } else {
                $this->view->error = $result->getErrorMessages();
            }
        }
        return $result;
    }
    
    protected
    function displayReuse()
    {
        $result = new Am_Paysystem_Result;
        $tr = new Am_Paysystem_Transaction_Recurly_FindAccount($this->plugin, $this->invoice);
        $tr->run($result);
        if (!$result->isSuccess()) {
            throw new Am_Exception_Paysystem("Stored customer profile not found");
        }
        $billingInfoTr = new Am_Paysystem_Transaction_Recurly_GetBillingInfo($this->plugin, $this->invoice);
        $billingInfoTr->run(new Am_Paysystem_Result());
        
        $billingInfo = $billingInfoTr->getBillingInfo();
        
        $card = 'XXXX XXXX XXXX ' . (string)$billingInfo->last_four;
        
        $text = ___('Click "Continue" to pay this order using stored credit card %s', $card);
        $continue = ___('Continue');
        $cancel = ___('Cancel');
        
        $action = $this->plugin->getPluginUrl('cc');
        $id = Am_Html::escape($this->_request->get('id'));
        $action = Am_Html::escape($action);
        
        $receipt = $this->view->partial('_receipt.phtml', ['invoice' => $this->invoice, 'di' => $this->getDi()]);
        $this->view->layoutNoMenu = true;
        $this->view->content .= <<<CUT
$receipt
<div class='am-reuse-card-confirmation'>
<p>$text</p>
<form method='get' action='$action'>
    <input type='hidden' name='id' value='$id' />
    <input type='submit' class='tb-btn tb-btn-primary' name='reuse_ok' value='$continue' />
    &nbsp;&nbsp;&nbsp;
    <input type='submit' class='tb-btn' name='reuse_cancel' value='$cancel' />
</form>
</div>
CUT;
        $this->view->display('layout.phtml');
    }
    
    public
    function ccAction()
    {
        $this->view->title = ___('Payment info');
        $this->view->display_receipt = true;
        $this->view->layoutNoMenu = true;
        $this->view->invoice = $this->invoice;
        // if we have credit card on file, we will try to use it but we
        // have to display confirmation first
        if ($this->invoice->getUser()->data()->get(Am_Paysystem_Recurly::TOKEN)) {
            $s = $this->getDi()->session->ns($this->plugin->getId());
            $s->setExpirationSeconds(60 * 30); // after 30 minutes we will reset the session
            //$s->ccConfirmed = !empty($s->ccConfirmed);
            if ($this->_request->get('reuse_ok')) {
                if (@$s->ccConfirmed === true) {
                    $result = $this->plugin->doBill($this->invoice, $this->invoice->isFirstPayment(),
                        $this->getDi()->CcRecordTable->createRecord());
                    if ($result->isSuccess()) {
                        return $this->_redirect($this->plugin->getReturnUrl());
                    } else {
                        $this->invoice->getUser()->data()
                            ->set(Am_Paysystem_Recurly::TOKEN, null)
                            ->set(Am_Paysystem_Recurly::CC_EXPIRES, null)
                            ->set(Am_Paysystem_Recurly::CC_MASKED, null)
                            ->update();
                        $this->view->error = $result->getErrorMessages();
                        $s->ccConfirmed = false; // failed
                    }
                }
            } elseif ($this->_request->get('reuse_cancel') || (@$s->ccConfirmed === false)) {
                $s->ccConfirmed = false;
            } elseif (@$s->ccConfirmed === true) {
                try {
                    return $this->displayReuse();
                } catch (Exception $e) {
                    // Ignore it.
                }
            }
        }
        
        $this->form = $this->createForm($this->invoice->second_total > 0 ? ___('Subscribe And Pay') : ___('Pay with Card'));
        $result = $this->ccFormAndSaveCustomer();
        if ($result->isSuccess()) {
            $result = $this->plugin->doBill($this->invoice, $this->invoice->isFirstPayment(),
                $this->getDi()->CcRecordTable->createRecord());
            if ($result->isSuccess()) {
                return $this->_redirect($this->plugin->getReturnUrl());
            } else {
                $this->invoice->getUser()->data()
                    ->set(Am_Paysystem_Recurly::TOKEN, null)
                    ->set(Am_Paysystem_Recurly::CC_EXPIRES, null)
                    ->set(Am_Paysystem_Recurly::CC_MASKED, null)
                    ->update();
                $this->view->error = $result->getErrorMessages();
            }
        }
        $public_key = json_encode($this->plugin->getConfig('api_public_key'));
        $this->view->headScript()->appendScript(<<<INITJS
jQuery(document).ready(function(){
    recurly.configure({$public_key});
});
INITJS
        );
        
        $this->view->headScript()->appendFile(Am_Paysystem_Recurly::URL_RECURLY_JS);
        $this->view->headLink()->appendStylesheet(Am_Paysystem_Recurly::URL_RECURLY_CSS);
        $this->view->headStyle()->appendStyle(<<<INITSTYLE
INITSTYLE
        );
        $this->view->form = $this->form;
        $this->view->display('cc/info.phtml');
    }
    
}

abstract
class Am_Paysystem_Transaction_Recurly extends Am_Paysystem_Transaction_CreditCard
{
    
    function createRequest($plugin, $action, $method)
    {
        $request = new Am_HttpRequest($plugin->getApiEndpoint($action), $method);
        $request->setAuth($plugin->getConfig('api_key'))
            ->setHeader('Content-Type', 'application/xml; charset=utf-8')
            ->setHeader('Accept', 'application/xml')
            ->setHeader('X-Api-Version', Am_Paysystem_Recurly::API_VERSION);
        return $request;
    }
    
    public
    function parseResponse()
    {
        $this->parsedResponse = simplexml_load_string($this->response->getBody());
    }
    
    abstract
    function getExpectedStatusCode();
    
    public
    function validate()
    {
        if ($this->response->getStatus() != $this->getExpectedStatusCode()) {
            if ((string)$this->parsedResponse->description) {
                $this->result->setFailed([(string)$this->parsedResponse->description]);
                return false;
            }
            foreach ($this->parsedResponse->error as $e) {
                $errors[] = (isset($e['field']) ? ((string)$e['field']) . " : " : "") . ((string)$e);
            }
            $this->result->setFailed($errors);
            return false;
        }
        $this->result->setSuccess($this);
        return true;
    }
    
    function processValidated()
    {
        // nop;
    }
    
}

/**
 * Convert temporary credit card profile to customer record
 */
class Am_Paysystem_Transaction_Recurly_CreateCustomer extends Am_Paysystem_Transaction_Recurly
{
    
    protected
        $parsedResponse = [];
    
    public
    function __construct(
        Am_Paysystem_Abstract $plugin,
        Invoice $invoice,
        $token
    ) {
        $request = $this->createRequest($plugin, 'accounts', 'POST');
        
        $account = new SimpleXMLElement('<account/>');
        $account->account_code = md5($invoice->getEmail());
        $account->first_name = $invoice->getFirstName();
        $account->last_name = $invoice->getLastName();
        $account->email = $invoice->getEmail();
        $account->billing_info->token_id = $token;
        $account->billing_info->currency = $invoice->currency;
        
        $request->setBody($account->asXML());
        parent::__construct($plugin, $invoice, $request, true);
    }
    
    public
    function getUniqId()
    {
        return (string)$this->parsedResponse->account_code;
    }
    
    public
    function getExpectedStatusCode()
    {
        return '201';
    }
    
}

class Am_Paysystem_Transaction_Recurly_UpdateCustomer extends Am_Paysystem_Transaction_Recurly
{
    
    protected
        $parsedResponse = [];
    
    public
    function __construct(
        Am_Paysystem_Abstract $plugin,
        Invoice $invoice,
        $token
    ) {
        $request = $this->createRequest($plugin, 'accounts/' . md5($invoice->getEmail()), 'PUT');
        
        $account = new SimpleXMLElement('<account/>');
        $account->account_code = md5($invoice->getEmail());
        $account->first_name = $invoice->getFirstName();
        $account->last_name = $invoice->getLastName();
        $account->email = $invoice->getEmail();
        $account->billing_info->token_id = $token;
        $account->billing_info->currency = $invoice->currency;
        
        $request->setBody($account->asXML());
        parent::__construct($plugin, $invoice, $request, true);
    }
    
    public
    function getUniqId()
    {
        return (string)$this->parsedResponse->account_code;
    }
    
    public
    function getExpectedStatusCode()
    {
        return '200';
    }
    
}

class Am_Paysystem_Transaction_Recurly_FindAccount extends Am_Paysystem_Transaction_Recurly
{
    
    public
    function __construct(
        Am_Paysystem_Abstract $plugin,
        Invoice $invoice
    ) {
        $request = $this->createRequest($plugin, 'accounts/' . md5($invoice->getEmail()), 'GET');
        parent::__construct($plugin, $invoice, $request, true);
    }
    
    public
    function getExpectedStatusCode()
    {
        return "200";
    }
    
    public
    function getUniqId()
    {
        return (string)$this->parsedResponse->account_code;
    }
    function getInfo()
    {
        return $this->parsedResponse;
    }
    
    
}

class Am_Paysystem_Transaction_Recurly_GetBillingInfo extends Am_Paysystem_Transaction_Recurly
{
    
    public
    function __construct(
        Am_Paysystem_Abstract $plugin,
        Invoice $invoice
    ) {
        $request = $this->createRequest($plugin, 'accounts/' . md5($invoice->getEmail()) . '/billing_info', 'GET');
        parent::__construct($plugin, $invoice, $request, true);
    }
    
    public
    function getExpectedStatusCode()
    {
        return "200";
    }
    
    public
    function getUniqId()
    {
        return (string)$this->parsedResponse->account_code;
    }
    
    function getBillingInfo()
    {
        return $this->parsedResponse;
    }
    
}

class Am_Paysystem_Transaction_Recurly_Transaction extends Am_Paysystem_Transaction_Recurly
{
    
    function __construct(\Am_Paysystem_Abstract $plugin, \Invoice $invoice, $doFirst)
    {
        $request = $this->createRequest($plugin, 'transactions', 'POST');
        
        $tr = new SimpleXMLElement('<transaction/>');
        $tr->amount_in_cents = ($doFirst ? $invoice->first_total : $invoice->second_total) * 100;
        $tr->currency = $invoice->currency;
        $tr->account->account_code = $invoice->getUser()->data()->get(Am_Paysystem_Recurly::TOKEN);
        $tr->description = $invoice->getLineDescription();
        if ($plan_code = $invoice->getItem(0)->getBillingPlanData('recurly_plan_code')) {
            $tr->product_code = $plan_code;
        }
        
        $request->setBody($tr->asXML());
        
        parent::__construct($plugin, $invoice, $request, $doFirst);
    }
    
    public
    function getExpectedStatusCode()
    {
        return "201";
    }
    
    public
    function getUniqId()
    {
        return (string)$this->parsedResponse->uuid;
    }
    
    public
    function validate()
    {
        if ($this->response->getStatus() != $this->getExpectedStatusCode()) {
            if ((string)$this->parsedResponse->description) {
                $this->result->setErrorMessages([(string)$this->parsedResponse->description]);
                return false;
            }
            foreach ($this->parsedResponse->error as $e) {
                $errors[] = (isset($e['field']) ? ((string)$e['field']) . " : " : "") . ((string)$e);
            }
            $this->result->setErrorMessages($errors);
            return false;
        } else {
            if (((string)$this->parsedResponse->status) != 'success') {
                $this->result->setErrorMessages([(string)$this->parsedResponse->transaction_error->customer_message]);
                return false;
            }
        }
        $this->result->setSuccess($this);
        return true;
    }
    
    function processValidated()
    {
        $this->invoice->addPayment($this);
    }
    
}

class Am_Paysystem_Transaction_Recurly_Refund extends Am_Paysystem_Transaction_Recurly
{
    
    public function __construct(Am_Paysystem_Abstract $plugin, Invoice $invoice, $charge_id, $amount = null)
    {
        $this->charge_id = $charge_id;
        $this->amount = $amount > 0 ? $amount : null;
        $request = $this->createRequest($plugin,
            'transactions/' . $this->charge_id . "?amount_in_cents=" . ($amount * 100), 'DELETE');
        parent::__construct($plugin, $invoice, $request, true);
    }
    
    public function getUniqId()
    {
        return $this->charge_id . '-RFND';
    }
    
    public function processValidated()
    {
        $this->invoice->addRefund($this, $this->charge_id, $this->amount);
    }
    
    public
    function getExpectedStatusCode()
    {
        return "204";
    }
    
}


class Am_Paysystem_Transaction_Recurly_Incoming extends Am_Paysystem_Transaction_Incoming
{
    protected $ip = [
        '50.18.192.88',
        '52.8.32.100',
        '52.9.209.233',
        '50.0.172.150',
        '52.203.102.94',
        '52.203.192.184',
        '172.31.15.223'
    ],
        $parsedRequest;
    
    /**
     *
     * @param type $plugin
     * @param Am_Mvc_Request $request
     * @param type $response
     * @param type $invokeArgs
     */
    function __construct($plugin, $request, $response, $invokeArgs)
    {
        parent::__construct($plugin, $request, $response, $invokeArgs);
        $body = $request->getRawBody();
        $this->parsedRequest = @simplexml_load_string($body);
    }
    
    public
    function getUniqId()
    {
        return (string)$this->parsedRequest->transaction->id;
    }
    
    public
    function validateSource()
    {
        //$this->_checkIp($this->ip);
        if (!$this->parsedRequest) {
            throw new Am_Exception_Paysystem_TransactionSource('Wrong XML unable to parse!');
        }
        
        if (!in_array($this->parsedRequest->getName(),
            ['successful_payment_notification', 'successful_refund_notification', 'closed_invoice_notification'])) {
            return false;
        }
        
        return true;
    }
    
    public
    function validateStatus()
    {
        if ($this->parsedRequest->getName() == 'closed_invoice_notification') {
            return (((string)@$this->parsedRequest->invoice->state) == 'collected');
        } else {
            return (((string)@$this->parsedRequest->transaction->status) == 'success');
        }
    }
    
    public
    function validateTerms()
    {
        return true;
    }
    
    function fetchUserInfo()
    {
        return [
            'email' => $this->parsedRequest->account->email,
            'name_f' => $this->parsedRequest->account->first_name,
            'name_l' => $this->parsedRequest->account->last_name,
        ];
    }
    
    function generateInvoiceExternalId()
    {
        return (string)($this->parsedRequest->transaction->subscription_id ?: ($this->parsedRequest->invoice->subscription_id ?: $this->parsedRequest->transaction->invoice_id));
    }
    
    function generateUserExternalId(array $userInfo)
    {
        return md5((string)$this->parsedRequest->account->email);
    }
    
    function autoCreateGetProducts()
    {
        $subscription_id = $this->generateInvoiceExternalId();
        $request = new Am_HttpRequest($this->getPlugin()->getApiEndpoint('subscriptions/' . $subscription_id), 'GET');
        $request->setAuth($this->getPlugin()->getConfig('api_key'))
            ->setHeader('Content-Type', 'application/xml; charset=utf-8')
            ->setHeader('Accept', 'application/xml')
            ->setHeader('X-Api-Version', Am_Paysystem_Recurly::API_VERSION);
        $resp = $request->send();
        if ($resp->getStatus() != '200') {
            throw new Am_Exception_Paysystem_TransactionInvalid('Unable to get info about subscirption');
        }
        $data = @simplexml_load_string($resp->getBody());
        if (!$data) {
            throw new Am_Exception_Paysystem_TransactionInvalid('Unable to get info about subscirption');
        }
        
        $billing_plan = $this->getPlugin()->getDi()->billingPlanTable->findFirstByData('recurly_plan_code',
            (string)$data->plan->plan_code);
        if ($billing_plan) {
            return [$billing_plan->getProduct()];
        }
        
    }
    
    function findInvoiceId()
    {
        return '';
    }
    
    function processValidated()
    {
        switch ((string)$this->parsedRequest->getName()) {
            case 'successful_payment_notification':
                $this->invoice->addPayment($this);
                break;
            case 'successful_refund_notification':
                $this->invoice->addRefund($this, (string)$this->parsedRequest->transaction->id);
                break;
            case 'closed_invoice_notification' :
                if (intval($this->parsedRequest->invoice->total_in_cents) == 0) {
                    $this->invoice->addAccessPeriod($this);
                }
                break;
        }
    }
}
