<?php

/**
 * @table paysystems
 * @id lexpay
 * @title LexPay
 * @desc Lex Pay checkout is the smarter way to sell your digital goods online
 * @visible_link https://lexholdingsgroup.com
 * @recurring none
 * @logo_url lexpay.png
 * @am_payment_api 6.0
 */
class Am_Paysystem_Lexpay extends Am_Paysystem_Abstract
{
    const PLUGIN_STATUS = self::STATUS_BETA;
    const PLUGIN_REVISION = '6.3.21';

    protected $defaultTitle = 'LexPay';
    protected $defaultDescription = 'Pay by LexPay';

    public function _initSetupForm(Am_Form_Setup $form)
    {
        $form->addSecretText('secret')
            ->setLabel('Secret Merchant Key')
            ->addRule('required')
            ->addRule('callback', 'Invalid Merchant Key', [$this, 'checkMerchantKey']);
    }

    function isConfigured()
    {
        return (bool) $this->getConfig('secret');
    }

    public function getRecurringType()
    {
        return self::REPORTS_NOT_RECURRING;
    }

    function checkMerchantKey($secret)
    {
        $req = new Am_HttpRequest("https://lexholdingsgroup.com/validate/{$secret}");
        $resp = $req->send();

        return $resp->getStatus() == 200;
    }

    public function _process($invoice, $request, $result)
    {
        $user = $invoice->getUser();

        $req = new Am_HttpRequest('https://lexholdingsgroup.com/create', Am_HttpRequest::METHOD_POST);
        $req->setBody(json_encode([
            'secret' => $this->getConfig('secret'),
            'email' => $user->email,
            'amount' => $invoice->first_total,
            'webhookURL' => $this->getPluginUrl('ipn'),
            'returnURL' => $this->getReturnUrl(),
            'metadata' => $invoice->public_id,
        ]));

        $log = $this->logRequest($req);
        $resp = $req->send();
        $log->add($resp);

        if ($data = json_decode($resp->getBody(), true)) {
            $log->add($data);
        }

        if ($resp->getStatus() != 200) {
            $result->setFailed('Incorrect HTTP response status: ' . $resp->getStatus());
            return;
        }

        $a = new Am_Paysystem_Action_Redirect("https://lexholdingsgroup.com/checkout/{$data['response']}");
        $result->setAction($a);
    }

    public function createTransaction($request, $response, array $invokeArgs)
    {
        return new Am_Paysystem_Transaction_Lexpay($this, $request, $response, $invokeArgs);
    }
}

class Am_Paysystem_Transaction_Lexpay extends Am_Paysystem_Transaction_Incoming
{
    protected $parsedResponse = null;

    function __construct($plugin, $request, $response, $invokeArgs)
    {
        parent::__construct($plugin, $request, $response, $invokeArgs);
        $this->parsedResponse = json_decode($request->getRawBody(), true);
    }

    protected function get($token, $default = '')
    {
        return $this->parsedResponse[$token] ?? $default;
    }

    public function getUniqId()
    {
        return $this->get('id');
    }

    public function findInvoiceId()
    {
        return $this->get('metadata');
    }

    public function validateSource()
    {
        return $this->plugin->getConfig('secret') === $this->get('secret');
    }

    public function validateStatus()
    {
        return $this->get('status') == 'paid';
    }

    public function validateTerms()
    {
        return true;
    }
}