import React, {useEffect} from "react";
import {TData} from "./types";
import {Container, Row} from "./AdminCategory/Components/Styled";
import CategoryForm from "./AdminCategory/Components/CategoryForm";
import {observer} from "mobx-react";
import {CategoryStore, CategoryStoreContext, TListStatus, useCategoryStore} from "./AdminCategory/Store";
import {TCategory} from "./AdminCategory/types";
import CategoryList from "./AdminCategory/Components/CategoryList";
import Loading from "./Utils/Loading";

const AdminCategoryContainer = observer(() => {
    const store = useCategoryStore();
    const onSubmit = (category: TCategory) => {
        store.closeForm()
    }
    return (
        <Container>
            <Row>
                {store.listStatus == TListStatus.IDLE && <CategoryList items={store.categoryTree}/>}
                {store.listStatus == TListStatus.LOADING && <Loading>loading</Loading>}
            </Row>
            <Row>
                <div>
                    <a href="#form" onClick={() => store.showForm("add")}>Add Category</a>
                </div>
                <CategoryForm show={store.currentForm === 'add'} formId={"add"} onSubmit={onSubmit}
                              onClose={() => store.closeForm()}/>
            </Row>
        </Container>

    );
})

const AdminCategory: React.FC<{ data: TData }> = ({data}) => {
    const store = new CategoryStore({uri: data.uri ?? "", type_id: data.type_id ?? "", max_upload_width: (data.max_upload_width??200) as number, max_upload_size:(data.max_upload_size??200) as number});
    useEffect(() => {
        store.apiLoad();
    }, [data]);

    return (<CategoryStoreContext.Provider value={store}>
            <AdminCategoryContainer/>
        </CategoryStoreContext.Provider>
    )
}


export default AdminCategory;