import React, {useState} from "react";
import {observer} from "mobx-react";
import {useCategoryStore} from "../Store";
import {TCategoryTreeItem} from "../types";
import {
    CategoryItemAction,
    CategoryItemActionsContainer,
    CategoryItemContainer,
    CategoryListContainer,
    ToggleChilds,
} from "./Styled";
import CategoryForm from "./CategoryForm";

type TItemProps = {
    item: TCategoryTreeItem
    onMoveUp: () => void
    onMoveDown: () => void
}
const Item: React.FC<TItemProps> = observer((props) => {
    const store = useCategoryStore()
    const category = store.categories[props.item.id];
    const editFormId = "edit-" + category.category_id
    const childFormId = 'child-' + category.category_id
    const childIds = props.item.children;
    const hasChilds = childIds.length > 0;
    const [showChilds, toggleShowChilds] = useState(false);
    const deleteCategory = () => {
        if (!hasChilds || window.confirm('Category is not empty. Are you sure?')) {
            store.apiDeleteCategory(category);
        }
    }


    return (<><CategoryItemContainer>
            {!store.isFormDisplayed(editFormId) && <>
                <div>
                    {hasChilds && <ToggleChilds onClick={() => toggleShowChilds(!showChilds)}>
                        {showChilds ? <i className="fas fa-angle-down"></i> :
                            <i className="fas fa-angle-right"></i>} {category.name}
                    </ToggleChilds>}
                    {!hasChilds && <><i className="fas fa-angle-right"
                                        style={{color: "transparent"}}></i> {category.name}</>}

                </div>
                <CategoryItemActionsContainer>
                    <CategoryItemAction title={"Move Up"} onClick={props.onMoveUp}><i
                        className="fas fa-arrow-up"></i></CategoryItemAction>
                    <CategoryItemAction title={"Move Down"} onClick={props.onMoveDown}><i
                        className="fas fa-arrow-down"></i></CategoryItemAction>
                    <CategoryItemAction title={"Add Subcategory"} onClick={() => store.showForm(childFormId)}><i
                        className="fas fa-plus"></i></CategoryItemAction>
                    <CategoryItemAction title={"Edit"} onClick={() => store.showForm(editFormId)}><i
                        className="fas fa-edit"></i></CategoryItemAction>
                    <CategoryItemAction title={"Delete"} onClick={deleteCategory}><i
                        className="fas fa-trash"></i></CategoryItemAction>
                </CategoryItemActionsContainer>
            </>}
            <CategoryForm show={editFormId === store.currentForm} formId={editFormId} onClose={() => store.closeForm()}
                          category={category}/>
            <CategoryForm show={childFormId === store.currentForm} formId={childFormId}
                          onClose={() => store.closeForm()} category={{parent_id: category?.category_id}}/>
        </CategoryItemContainer>
            {hasChilds && showChilds && <CategoryList items={props.item.children}/>}
        </>
    );
})

const CategoryList = observer((props: { items: Array<TCategoryTreeItem> }) => {
    const store = useCategoryStore();
    return (<>
        <CategoryListContainer>
            {props.items.map((el, i) => <Item key={i} item={el} onMoveUp={() => store.apiMoveUp(props.items, el.id)}
                                              onMoveDown={() => store.apiMoveDown(props.items, el.id)}/>)}
        </CategoryListContainer>
    </>)
});
export default CategoryList;