import React, {FC} from "react"
import {TChatLayoutProps} from "../types";
import {Alert, Grid} from "@mui/material";
import {observer} from "mobx-react";
import {useChatStore} from "../Store";
import ChatWindow from "./Window";
import ChatInputBox from "./InputBox";

const ChatLayout: FC<TChatLayoutProps> = observer(() => {

    const store = useChatStore();

    return <>
        <Grid container flexDirection={"column"} rowSpacing={1}>
            <Grid item>
                <ChatWindow messages={store.messages} onNeedMore={() => store.apiGetHistory()} total={store.total ?? 0}
                            ownerId={store.ownerId}/>
            </Grid>
            <Grid item>
                {store.error && <Alert severity={"error"}>{store.error}</Alert>}
                <ChatInputBox onMessagePost={(message) => store.apiPostMessage(message)}/>
            </Grid>
        </Grid>
    </>
})
export default ChatLayout;

