import React, {FC} from "react"
import {Alert, Container, Grid} from "@mui/material";
import MarketplaceLayoutNavigation from "./Layout/Navigation";
import {useMarketplaceStore} from "./Store";
import {observer} from "mobx-react";
import Loading from "../Utils/Loading";
import useNavigationTabs from "./Store/tabs";
import {TMarketplaceLayoutProps} from "./types";


const MarketplaceLayout: FC<TMarketplaceLayoutProps> = observer((props) => {
    const tabs = useNavigationTabs();
    const rootStore = useMarketplaceStore();
    return (
        <Container>
            {rootStore.isError ? <Alert severity={"error"}>{rootStore.errorMessage}</Alert> :
                <Grid container spacing={2}>
                    <Grid item xs={12}><MarketplaceLayoutNavigation tabs={tabs} filter={props.filter}/></Grid>
                    <Grid item xs={12}>
                        <React.Suspense fallback={<Loading/>}>
                            {props.children}
                        </React.Suspense>
                    </Grid>
                </Grid>
            }
        </Container>

    );
})
export default MarketplaceLayout;