import React, {FC, useEffect, useState} from "react";
import {useDropzone} from "react-dropzone";
import {Badge, Grid, Typography, useTheme} from "@mui/material";
import {TImageUploadPros} from "../types";
import Img from "./Listing/Img";


const ImageUpload: FC<TImageUploadPros> = (props) => {
    const theme = useTheme();
    const [preview, setPreview] = useState<string>();
    const {acceptedFiles, getRootProps, getInputProps} = useDropzone({
        maxSize: props.maxUploadSize * 1024,
        onDrop: (acceptedFiles, fileRejections, event) => {
            if (acceptedFiles[0] !== undefined) {
                if(['image/png', 'image/jpeg', 'image/gif'].includes(acceptedFiles[0].type)){
                    props.onSetFile(acceptedFiles[0]);
                    setPreview(URL.createObjectURL(acceptedFiles[0]));
                }
            }
        }
    });

    useEffect(() => {
        setPreview(props.url);
    }, [props.url]);
    return (
        <Grid container sx={{
            border: `2px dotted ${theme.palette.secondary.light}`,
            padding: "1em"
        }} {...getRootProps({className: 'dropzone'})}>
            {preview && <Grid item xs={4}>
                <Badge sx={{cursor: "pointer"}} badgeContent={"X"} color="primary" onClick={(event) => {
                    event.preventDefault();
                    event.stopPropagation();
                    setPreview(undefined);
                    props.onDeleteFile();
                }}>
                    <Img src={preview} width={props.maxUploadWidth}/>
                </Badge>

            </Grid>}
            <Grid item xs>
                <input {...getInputProps()} />
                <Typography color={"secondary"}>Attach an image to listing</Typography>
                <Typography color={"secondary"} variant={"body2"}>Drag 'n' drop file here, or click to select
                    file</Typography>
                <Typography color={"secondary"} variant={"body2"}>One image file accepted(jpg, png, gif).
                    Max {props.maxUploadSize}Kb</Typography>
            </Grid>
        </Grid>
    );

}
export default ImageUpload;