import React, {FC, useEffect, useState} from "react";
import {Alert, Button, Grid, Link, Typography} from "@mui/material";
import Quantity from "../Elements/Quantity";
import Loading from "../../../Utils/Loading";
import {observer} from "mobx-react";
import {TOrderSidebarProps, TPurchaseListingData} from "../../types";
import Receipt from "../Receipt";


const OrderSidebar: FC<TOrderSidebarProps> = observer((props) => {
    const [userCurrentBalance, setUserCurrentBalance] = useState(0);
    const [inPurchase, togglePurchase] = useState(false);
    const [error, setError] = useState("");
    const [total, setTotal] = useState(0);
    const [qty, setQty] = useState(1);
    const userBalance = props.currentUser?.balance ?? 0;

    const updateTotal = (qty: number) => {
        setQty(qty);
        let total = (props.listing.price ?? 0) * qty
        setTotal(total)
        if (props.currentUser) {
            setUserCurrentBalance(props.currentUser.balance - total)
        }

    }

    useEffect(() => {
        updateTotal(qty ?? 1);
    }, [props.listing, props.currentUser]);


    const purchaseListing = (data: TPurchaseListingData) => {
        togglePurchase(true);
        props.onPurchaseListing(data)
            .catch((msg) => {
                setError(msg)
            }).finally(() => {
            togglePurchase(false);
        })
    }

    return <Grid item container xs={6} sx={{paddingLeft: "1em", paddingRight: "1em"}} alignItems={"flex-start"}
                 flexDirection={"column"} columnSpacing={2}>
        <Receipt
            price={props.listing.price}
            qty={props.listing.qty > 1 ? <Quantity initial={1}
                                                   max={Math.min(props.listing.qty, Math.floor((userBalance) / props.listing.price))}
                                                   onChange={(qty) => updateTotal(qty)}/> :
                <Typography variant={"body2"}>1</Typography>}
            total={total}
            qtyTitle={`Quantity (available: ${props.listing.qty} )`}
            delivery={props.listing.deliveryText}
            deliveryTitle={"Delivery Speed, hours"}
        />


        <Grid item container marginBottom={"1em"}>
            {error && <Grid item xs={12} marginBottom={"1em"}><Alert severity={"error"}>{error}</Alert></Grid>}
            <Grid item xs={12}>
                <Button
                    variant={"contained"}
                    fullWidth
                    disabled={!props.listing.status.active || inPurchase}
                    color={props.listing.status.active ? "primary" : (props.listing.status.purchased ? "success" : "secondary")}
                    onClick={() => purchaseListing({listing_id: parseInt(props.listing.id), qty: qty})}
                >
                    Purchase
                    {inPurchase ? <Loading/> : " "}
                </Button></Grid>
        </Grid>
        {props.currentUser && <Grid item container>
            <Grid item container alignItems={"flex-start"} justifyContent={"space-between"}
                  sx={{marginBottom: "0.5em"}}>
                <Grid item>
                    <Typography variant={"body2"} component={"span"}>Your Credits Balance <Link
                        href={props.buyCreditsUrl} fontSize={"small"}>buy more</Link></Typography>

                </Grid>
                <Grid item>
                    <Typography variant={"body2"}>{userBalance}</Typography>
                </Grid>
            </Grid>
            <Grid item container alignItems={"flex-start"} justifyContent={"space-between"}
                  sx={{marginBottom: "0.5em"}}>
                <Grid item>
                    <Typography variant={"body2"}>Balance After Order</Typography>
                </Grid>
                <Grid item>
                    <Typography variant={"body2"}>{userCurrentBalance}</Typography>
                </Grid>
            </Grid>

        </Grid>}


    </Grid>
})
export default OrderSidebar