import React, {FC, ReactNode, useState} from "react";
import {Grid, Tab, Tabs} from "@mui/material";
import {Link, useLocation} from "react-router-dom";
import {TNavigationTab} from "../types";
import {observer} from "mobx-react";


const MarketplaceLayoutNavigation: FC<{ tabs: Array<TNavigationTab>, filter: ReactNode }> = observer((props) => {
    const location = useLocation();
    const initialState = props.tabs.sort((a, b) => a.value - b.value).reduce<number | undefined>((prev, curr, i) => {
        if (prev !== undefined) return prev
        return curr.regex.test(location.pathname) ? curr.value : prev
    }, undefined) ?? 1;

    const [value, setValue] = useState(initialState);


    return (<Grid container alignItems={"center"}>
            <Grid item flexGrow={1} xs={6}>
                <Tabs value={value} onChange={(event, newValue) => setValue(newValue)} aria-label="nav tabs example">
                    {props.tabs.sort((a, b) => a.order - b.order).map((el, i) =>
                        <Tab icon={el.icon ?? undefined} label={el.label ?? undefined} value={el.value} to={el.to}
                             component={Link} key={i}/>
                    )}
                </Tabs>
            </Grid>
            {props.filter}
        </Grid>
    )
})
export default MarketplaceLayoutNavigation;
