import React, {FC, useState} from 'react'
import {TTransactionSidebarProps} from "../../types";
import {Alert, Button, Grid, styled} from "@mui/material";
import Loading from "../../../Utils/Loading";
import ConfirmDialog from "../ConfirmDialog";
import TransactionInfo from "./Info";
import Header from "./Header";
import {useMarketplaceStore} from "../../Store";
import ChatWidget from '../../../Chat/ChatWidget';
import TransactionSerialCodes from "./SerialCodes";

const Description = styled("pre")({
    whiteSpace: "pre-wrap"
});

const TransactionSidebar: FC<TTransactionSidebarProps> = (props) => {
    const [comment, setComment] = useState(props.transaction.comment);
    const [commentUpdating, toggleCommentUpdating] = useState(false);
    const [commentUpdateError, setCommentUpdateError] = useState("");
    const [showCommentDialog, toggleCommentDialog] = useState(false);
    const [showFinishSaleDialog, toggleFinishSaleDialog] = useState(false);
    const [processingSale, toggleProcessingSale] = useState(false);
    const [processingError, setProcessingError] = useState("");
    const rootStore = useMarketplaceStore();

    const updateComment = (comment: string) => {
        toggleCommentDialog(false);
        toggleCommentUpdating(true);
        setCommentUpdateError("");

        if (!comment)
            setCommentUpdateError("Comment is empty");
        else
            props.onCommentUpdate(comment).then(() => {

            })
                .catch((e: string) => {
                    setCommentUpdateError(e);
                }).finally(() => {
                toggleCommentUpdating(false);
            })
    }
    const completeSale = () => {
        toggleFinishSaleDialog(false);
        toggleProcessingSale(true)
        setProcessingError("")
        props.onSaleConfirming().then(() => {

        })
            .catch((e: string) => setProcessingError(e))
            .finally(() => {
                toggleProcessingSale(false);
            })


    }
    return <Grid item container xs={6} sx={{paddingLeft: "1em", paddingRight: "1em"}} alignItems={"flex-start"}
                 flexDirection={"column"} columnSpacing={2}>

        <TransactionInfo transaction={props.transaction}/>

        <Grid item container direction={"column"}>
            <Header>Download Instructions</Header>
            <Grid item sx={{paddingTop: '0.5em'}}>
                <Description>{props.transaction.listing.instructions}</Description>
            </Grid>
        </Grid>
        <TransactionSerialCodes
            show={props.transaction.serial_codes != undefined && props.transaction.status == 'completed'}
            codes={props.transaction.serial_codes ?? []}/>
        {['pending', 'sent'].includes(props.transaction.status) &&
            <Grid item container sx={{padding: '1em 0 1em 0'}} direction={"column"}>
                {processingError &&
                    <Grid item sx={{marginBottom: "0.5em"}}><Alert severity={"error"}>{processingError}</Alert></Grid>}
                <Grid item>
                    <Button
                        fullWidth
                        variant={"contained"}
                        onClick={() => toggleFinishSaleDialog(true)}
                        disabled={processingSale}
                    >
                        Mark Sale As Complete {processingSale && <Loading/>}
                    </Button>
                </Grid>
                <ConfirmDialog
                    open={showFinishSaleDialog}
                    onClose={() => toggleFinishSaleDialog(false)}
                    title={"Confirm Item Received"}
                    onConfirmed={() => completeSale()}
                    okBtnText={"CONFIRM"}
                >
                    Confirm sale only if you have received item from seller.

                </ConfirmDialog>

            </Grid>}
        <Grid item container direction={"column"}>
            <Header>Chat With Seller</Header>
            <Grid item sx={{paddingTop: "0.5em"}}>
                <ChatWidget chatId={props.transaction.transaction_id} ownerId={props.transaction.buyer_id}
                            apiUrl={rootStore.apiUrl('/transaction/chat')}/>
            </Grid>

        </Grid>


    </Grid>
}
export default TransactionSidebar;
