import {MarketplaceStore} from "./index";
import {makeAutoObservable, runInAction} from "mobx";
import {TTransaction} from "../types";
import {TApiResponseList} from "../../types";

export default class TransactionsStore {
    list: Array<TTransaction> = []
    rootStore: MarketplaceStore
    filter: string = ""
    type: "purchased" | "sold" = 'purchased'
    total: number = 0

    constructor(rootStore: MarketplaceStore) {
        this.rootStore = rootStore
        makeAutoObservable(this, {
            rootStore: false
        });
    }

    async apiGetTranactions(params: { type: string, skipFirst?: number, id?: string }) {
        const response = await this.rootStore.apiRequest('/transaction/list', {
            skipFirst: params?.skipFirst ?? 0,
            type: params.type,
            filter: this.filter,
            id: params?.id

        });
        if (response.ok == true) {
            runInAction(() => {
                    let r = response as TApiResponseList<TTransaction>;
                    this.total = r.total
                    this.list = (params?.skipFirst !== undefined) ? [...this.list, ...r.list] : r.list;
                }
            )

        }
        return response as TApiResponseList<TTransaction>;

    }

    setFilter(text: string) {
        this.filter = text
    }

}