import {computed, makeAutoObservable, runInAction} from "mobx";
import {MarketplaceStore} from "./index";
import {TCurrentUser, TListingFormValue, TTransaction} from "../types";
import {ListingRecord} from "./Listing";

export default class UserStore {
    rootStore: MarketplaceStore;
    private currentUser: TCurrentUser | null = null;
    isError: boolean = false;
    errorMessage: string = "";

    constructor(rootStore: MarketplaceStore) {
        this.rootStore = rootStore;
        makeAutoObservable(this, {
            user: computed,
            historyUpdates: computed
        });
    }

    async fetchUser(): Promise<any> {
        const response = await this.rootStore.apiRequest('/me');
        if (response.ok == true) {
            runInAction(() => {
                this.updateUser((response as any).currentUser)
            })
        }
        return Promise.resolve();
    }

    updateUser(user: TCurrentUser) {
        if (!this.currentUser) {
            this.currentUser = user;
        } else {
            Object.assign(this.currentUser, user);
        }

    }

    setTransactionViewed(id: number) {
        if (this.currentUser) {
            this.currentUser.historyUpdates.sales = this.currentUser.historyUpdates.sales.filter((item) => item != id);
            this.currentUser.historyUpdates.orders = this.currentUser.historyUpdates.orders.filter((item) => item != id);
        }
    }

    get historyUpdates() {

        return this.currentUser ? this.currentUser.historyUpdates : {sales: [], orders: []};
    }

    get user() {
        return this.currentUser;
    }

    async getHistoryUpdates(): Promise<any> {
        const response = await this.rootStore.apiRequest('/history-updates');
        if (response.ok == true) {
            runInAction(() => {
                if (this.currentUser) {
                    this.currentUser.historyUpdates.sales = [...this.currentUser.historyUpdates.sales, ...response.sales];
                    this.currentUser.historyUpdates.orders = [...this.currentUser.historyUpdates.orders, ...response.orders];

                }
            });
        }
        return Promise.resolve();

    }

    get unseenSales() {
        return this.currentUser?.historyUpdates.sales ?? [];
    }

    get unseenOrders() {
        return this.currentUser?.historyUpdates.orders ?? [];
    }

    isTransactionNew(id: number) {
        return this.unseenSales.includes(id) || this.unseenOrders.includes(id);
    }

    isMineListing(listing: ListingRecord) {
        return this.currentUser?.login == listing.owner.username;
    }

    isSeller(transaction: TTransaction) {
        return parseInt(this.currentUser?.user_id ?? "") == transaction.seller_id;
    }

    isBuyer(transaction: TTransaction) {
        return parseInt(this.currentUser?.user_id ?? "") == transaction.buyer_id;
    }

    canEdit(listing: TListingFormValue) {
    }

    isLoggedIn() {
        return this.currentUser != null;
    }


}


