import React, {useEffect, useState} from "react";
import {observer} from "mobx-react";
import MarketplaceLayout from "../Layout";
import {useMarketplaceStore} from "../Store";
import {Backdrop, Grid, Typography, useTheme} from "@mui/material";
import {LoadingBox} from "../../Utils/Loading";
import TransactionsFilter from "../Layout/Transaction/Filter";
import OwnerInfo from "../Layout/Listing/OwnerInfo";
import TransactionStatus from "../Layout/Transaction/Status";
import {Link, useParams} from "react-router-dom";

const InfiniteScroll = React.lazy(() => import("react-infinite-scroller"));


const HistoryGrid = observer(() => {
    const store = useMarketplaceStore();
    const [loading, setLoading] = useState(false);
    const theme = useTheme();
    const {type = 'orders'} = useParams();
    useEffect(() => {
        setLoading(true);
        store.transactions.apiGetTranactions({type: type}).then(() => {

        }).finally(() => {
            setLoading(false)
        })

    }, [store.transactions.filter, store.transactions.type]);


    return <MarketplaceLayout filter={<TransactionsFilter/>}>
        {loading ? <LoadingBox/> : <Grid container direction={"column"}>
            <Grid container item sx={{borderBottom: `1px solid ${theme.palette.secondary.light}`}}>
                <Grid item xs={2}>
                    <Typography variant={"body2"} color={"secondary"}>Updated</Typography>
                </Grid>
                <Grid item xs={6}>
                    <Typography variant={"body2"} color={"secondary"}>Item</Typography>
                </Grid>
                <Grid item xs={1}>
                    <Typography variant={"body2"} color={"secondary"}>Total</Typography>
                </Grid>

                <Grid item xs={2}>
                    <Typography variant={"body2"}
                                color={"secondary"}>{type == 'orders' ? "Seller" : "Buyer"}</Typography>
                </Grid>

                <Grid item xs={1}>
                    <Typography variant={"body2"} color={"secondary"}>Status</Typography>
                </Grid>
            </Grid>
            <Backdrop
                sx={{color: '#fff', zIndex: (theme) => theme.zIndex.drawer + 1, position: 'absolute'}}
                open={loading}
                invisible={true}
            >
                <LoadingBox size={2}/>
            </Backdrop>
            <InfiniteScroll
                loadMore={() => store.transactions.apiGetTranactions({
                    type: type,
                    skipFirst: store.transactions.list.length
                })}
                hasMore={store.transactions.total > store.transactions.list.length}
                loader={<LoadingBox size={2} key={0}/>}
            >
                {store.transactions.list.map((el, i) => {
                    let weight = store.userStore.isTransactionNew(el.transaction_id) ? "bold" : ""
                    return <Grid item container key={i} sx={{paddingTop: "1em"}}>
                        <Grid item xs={2}>
                            <Typography variant={"body2"} fontWeight={weight}>{el.updated}</Typography>
                        </Grid>
                        <Grid item xs={6}>
                            <Typography variant={"body2"} fontWeight={weight}><Link
                                to={`/history/${type}/${el.transaction_id}`}>{el.listing.title}</Link></Typography>
                        </Grid>
                        <Grid item xs={1}>
                            <Typography variant={"body2"} fontWeight={weight}>{el.total}</Typography>
                        </Grid>

                        <Grid item xs={2}>
                            <Typography variant={"body2"} fontWeight={weight}>{type == 'orders' ?
                                <OwnerInfo record={el.listing}/> : el.buyer}</Typography>
                        </Grid>
                        <Grid item xs={1}>
                            <TransactionStatus status={el.status}
                                               fontWeight={weight}>{el.statusText}</TransactionStatus>
                        </Grid>

                    </Grid>
                })}
            </InfiniteScroll>
        </Grid>
        }
    </MarketplaceLayout>
})

export default HistoryGrid
