import React, {FC, useEffect, useState} from "react";
import {observer} from "mobx-react";
import MarketplaceLayout from "../Layout";
import {useNavigate, useParams} from "react-router-dom";
import {Button, Grid, useTheme} from "@mui/material";
import {useCurrentUser, useMarketplaceStore} from "../Store";
import {ListingRecord} from "../Store/Listing";
import {LoadingBox} from "../../Utils/Loading";
import {TApiResponseWithStatus} from "../../types";
import {TPurchaseListingData, TTransaction} from "../types";
import OrderSidebar from "../Layout/Listing/OrderSidebar";
import ListingItem from "../Layout/Listing/Item";
import LoginButton from "../Layout/Elements/LoginSidebar";


const Item: FC<{ type: string }> = observer((props) => {
    const {id} = useParams();
    const navigate = useNavigate();
    const store = useMarketplaceStore();
    const [listing, setListing] = useState<ListingRecord>();
    const [loading, toggleLoading] = useState(false);
    const currentUser = useCurrentUser();

    const theme = useTheme();
    const apiPurchase = (data: TPurchaseListingData) => {
        const postData = Object.assign(data, {selfUrl: store.selfUrl})
        return store.apiRequest('/listing/purchase', postData).then((response) => {
            let ret = response as TApiResponseWithStatus
            if (ret.status == 'error') {
                return Promise.reject(ret.error ?? "");
            } else if (ret.status == 'redirect' && ret.url) {
                document.location.href = ret.url
            } else {
                let transaction = (response as any).transaction as TTransaction
                navigate(`/history/orders/${transaction.transaction_id}`);
                store.userStore.fetchUser();
            }
        })
    }
    useEffect(() => {
        if (id !== undefined) {
            toggleLoading(true);
            store.listingStore.apiGetSingleListing(id, props.type).then((listing) => {
                setListing(listing);
            }).catch((msg) => {
                navigate("/");
            }).finally(() => toggleLoading(false));

        } else {
            navigate("/");
        }
    }, [id])


    return <>
        <MarketplaceLayout>
            <Grid container direction={"column"}>
                <Grid item>
                    <Button onClick={() => navigate(-1)}>&lt;&lt;&lt; back</Button>
                </Grid>
                {loading && <LoadingBox/>}
                {listing && <ListingItem
                    listing={listing}
                    sidebar={currentUser ? <OrderSidebar
                        listing={listing}
                        currentUser={currentUser}
                        onPurchaseListing={apiPurchase}
                        buyCreditsUrl={store.config.buyCreditsUrl}
                    /> : <LoginButton/>}
                />}
            </Grid>
        </MarketplaceLayout>
    </>
})
export default Item;