import React, {useEffect, useState} from "react";
import {Box, Button, Grid} from "@mui/material";
import {NavigateFunction, useNavigate, useSearchParams} from "react-router-dom";
import {observer} from "mobx-react";
import {useListingStore, useMarketplaceStore} from "../Store";
import MarketplaceLayout from "../Layout";
import FilterBar from "../Layout/FilterBar";

const ListingsGrid = React.lazy(() => import("../Layout/ListingsGrid"));

const AddButton = ({navigate}: { navigate: NavigateFunction }) => {

    return <Box sx={{paddingBottom: "1em;"}}>
        <Button onClick={(event) => {
            event.preventDefault();
            navigate("/my/add", {replace: true})
        }}>
            Add Listing
        </Button></Box>
}
const MyListings = observer(() => {
    const navigate = useNavigate();
    const [isLoading, toggleLoading] = useState(false);
    const store = useMarketplaceStore();
    const listingStore = useListingStore();
    const [searchParams, setUrlSearchParams] = useSearchParams();

    useEffect(() => {
        //listingStore.setFilter(searchParams);
        toggleLoading(true);
        listingStore.apiGetListings({all: false, filter: listingStore.getFilter(searchParams)})
            .finally(() => {
                toggleLoading(false)
            })
    }, [searchParams, listingStore.sort]);

    const deleteListing = (id: string) => {
        toggleLoading(true);
        store.apiRequest('/listing/delete', {listing_id: id}).then((response) => {
            return store.listingStore.apiGetListings({all: false, filter: listingStore.getFilter(searchParams)});
        }).finally(() => {
            toggleLoading(false);
        })


    }
    return <>
        <MarketplaceLayout filter={<FilterBar/>}>
            <Grid container direction={"column"}>

                <Grid item xs={12}>
                    <AddButton navigate={navigate}/>
                </Grid>
                <Grid item xs={12}>
                    <ListingsGrid records={listingStore.list} isLoading={isLoading} showOrder={false} showOwner={false}
                                  onDelete={deleteListing}
                                  canEdit={true}
                                  canDelete={true}
                                  onNeedMoreRecords={() => listingStore.apiGetListings({
                                      all: false,
                                      skipFirst: listingStore.list.length,
                                      filter: listingStore.getFilter(searchParams)
                                  })}
                                  hasMoreRecords={listingStore.list.length < listingStore.totalRecords}
                    />
                </Grid>
            </Grid>
        </MarketplaceLayout>
    </>
})

export default MyListings;