import React from "react";
import {TListing, TListingFormError} from "./Marketplace/types";

type TData = Record<string, string | undefined>

type TBlockConfig = {
    id: string,
    component: React.FC<{ data: TData }>
}

export type TApiErrorResponse = {
    ok: false
    errorMessage: string
    errorCode: number
}

export type TApiSuccessResponse = {
    ok: true
    [key: string]: string | Record<string, string>
}

export type TApiResponseSaveForm = {
    status: "ok" | "error",
    error?: TListingFormError
    listing?: TListing
}
export type TApiResponseWithStatus = {
    status: "ok" | "error" | "redirect"
    error?: string
    url?: string
}

export type TApiResponseList<T> = {
    list: Array<T>
    total: number
}


export type TApiResponse =
    TApiErrorResponse
    | TApiSuccessResponse
    | (TApiSuccessResponse & (TApiResponseSaveForm | TApiResponseList | TApiResponseWithStatus))

