<?php declare(strict_types=1);

namespace Am\Marketplace\Slim\Controller;

use Am\Marketplace\Slim\Controller;
use Am\Marketplace\Slim\Response;
use MarketplaceTransaction;
use Psr\Http\Message\ServerRequestInterface;

class TransactionChat extends Controller
{
    
    function __invoke(ServerRequestInterface $request, Response $response, array $args): Response
    {
        $params = $request->getParsedBody();
        
        if (empty($params['transaction_id'])) {
            return $response->withJsonError(___('Empty Transaction'));
        }
        
        /**
         * @var MarketplaceTransaction $transaction
         */
        $transaction = $this->getModule()->transactionTable()->load($params['transaction_id']);
        
        if (!$transaction->isOwner($request->getAttribute('user'))) {
            return $response->withJsonError(___("You can't access this chat"));
        }
        
        
        $messages = isset($params['history']) ? $transaction->chat()->getHistory($params['last_id'] ?? null,
            $params['count'] ?? 40) : $transaction->chat()->getLatestMessages($params['last_id'] ?? null,
            $params['count'] ?? 40);
        
        $transaction->recordTransactionUpdate($request->getAttribute('user'), true);
        
        return $response->withJson([
            'ok' => true,
            'list' => $messages,
            'total' => $transaction->chat()->getTotal()
        ]);
        
    }
}