<?php declare(strict_types=1);

namespace Am\Marketplace\Slim\Controller;

use Am\Marketplace\Slim\Controller;
use Am\Marketplace\Slim\Response;
use Am_Image;
use Am_Upload;
use Am_Upload_Acl;
use Bootstrap_Marketplace;
use Psr\Http\Message\ServerRequestInterface;
use Upload as UploadFile;

class Upload extends Controller
{
    
    function __invoke(ServerRequestInterface $request, Response $response, array $args): Response
    {
        if (!$this->getDi()->uploadAcl->checkPermission(Bootstrap_Marketplace::UPLOAD_PREFIX,
            Am_Upload_Acl::ACCESS_WRITE,
            $this->getDi()->auth->getUser()) && !$this->getDi()->authAdmin->getUser()) {
            return $response->withJsonError(___("Access denied!"));
        }
        
        
        $upload = new Am_Upload($this->getDi());
        $upload->setPrefix(Bootstrap_Marketplace::UPLOAD_PREFIX);
        $upload->processSubmit('upload', false);
        /**
         * @var UploadFile $file ;
         */
        
        [$file] = $upload->getUploads();
        if (!$file) {
            return $response->withJson([
                'ok' => true,
                'status' => 'error',
                'error' => 'No files uploaded'
            ]);
        }
        
        
        try {
            $image = new Am_Image($file->getFullPath(), $file->mime);
            $image->resize(
                $this->getModule()->getConfig('image_width', 200), 0, Am_Image::RESIZE_FITWIDTH,
                Am_Image::FILL_TRANSPARENT)
                ->save($file->getFullPath(), $file->mime);
            
            return $response->withJson([
                'ok' => true,
                'name' => $file->getName(),
                'size_readable' => $file->getSizeReadable(),
                'upload_id' => $file->pk(),
                'mime' => $file->mime
            ]);
        } catch (Am_Exception $e) {
            return $response->withJsonError(___('No files uplaoded'));
        }
        
    }
}