<?php

abstract class Am_Mvc_Controller_Listings extends Am_Mvc_Controller_Grid
{
    use Am_Marketplace_GetModule;
    
    function checkAdminPermissions(Admin $admin)
    {
        return $admin->hasPermission(Bootstrap_Marketplace::ADMIN_PERM) || $admin->isSuper();
    }
    
    function preDispatch()
    {
        parent::preDispatch(); // TODO: Change the autogenerated stub
        if ($this->getParam("user_id")) {
            $this->layout = "admin/user-layout.phtml";
        }
    }
    
    function createDatasource(): Am_Query
    {
        $user_id = $this->getParam('user_id');
        
        $ds = new Am_Query($this->getModule()->listingsTable());
        $ds->leftJoin('?_user', 'u', 'u.user_id=t.user_id');
        $ds->addField('u.login', 'user_login');
        $ds->addField('u.user_id', 'user_id');
        $ds->addField('u.marketplace_rating', 'user_rating');
        $ds->addField('u.marketplace_transactions', 'user_transactions');
        if (!empty($user_id)) {
            $ds->addWhere('t.user_id=?', $user_id);
        }
        
        $i = 1;
        foreach ($this->getModule()->categoryTypeTable()->findBy([]) as $type) {
            $ds->leftJoin("?_marketplace_listing_category", "lc{$i}",
                "t.listing_id = lc{$i}.listing_id and lc{$i}.type_id='{$type->type_id}'");
            $ds->addField("lc{$i}.category_id", 'category_' . $type->type_id);
            $i++;
        }
        $ds->setOrder('listing_id', true);
        return $ds;
    }
    
    function setupFields(Am_Grid_Editable $grid): void
    {
        $grid->addField('listing_id', ___('ID'));
        $grid->addField('title', ___('Title'));
        $grid->addField('status', ___('Status'))->setFormatFunction([MarketplaceListingTable::class, 'getStatusText']);
        $grid->addField('created', ___('Created'))->setFormatFunction('amDateTime');
        $grid->addField('user', ___('User'))->setRenderFunction(function ($record)
        {
            return sprintf("<td><a href='%s' target='_top'>%s</a> (%s%% %s sold)</td>",
                $this->getDi()->surl("admin-users?_u_a=edit&_u_id={$record->user_id}"), $record->user_login,
                $record->user_rating ?? 0, $record->user_transactions ?? 0);
        });
    }
    
    abstract function gridId(): string;
    
    abstract function gridTitle(): string;
    
    function createGrid()
    {
        $grid = new Am_Grid_Editable($this->gridId(), $this->gridTitle(), $this->createDatasource(),
            $this->getRequest(), $this->getView());
        
        $this->setupFields($grid);
        $grid->actionDelete('insert');
        $grid->setForm([$this, 'createForm']);
        $grid->setFilter(new Am_Grid_Filter_Listings());
        $grid->addCallback(Am_Grid_Editable::CB_VALUES_TO_FORM, function (&$values, $record)
        {
            foreach ($this->getDi()->db->selectPage($total,
                "select * from ?_marketplace_listing_category c where listing_id=?",
                $record->listing_id) as $category) {
                
                $values["_category_" . $category['type_id']] = $category['category_id'];
            }
        });
        $grid->addCallback(Am_Grid_Editable::CB_AFTER_SAVE, function (&$values, $record)
        {
            $cats = [];
            foreach ($values as $k => $v) {
                if (strpos($k, '_category_') !== false) {
                    $type_id = substr($k, 10);
                    $cats[$type_id] = $v;
                }
            }
            $record->setCategories($cats);
        });
        return $grid;
    }
    
    function createForm()
    {
        $form = new Am_Form_Admin('_edit_listing');
        
        $form->addText('title', ['class' => 'am-el-wide'])->setLabel(___('Listing Title'));
        
        $form->addTextarea('description', ['class' => 'am-el-wide', 'rows' => 5])->setLabel(___('Description'));
        
        $form->addSelect('listing_type')
            ->setLabel(___('Listing Type'))
            ->loadOptions([
                MarketplaceListing::TYPE_REGULAR => 'Regular',
                MarketplaceListing::TYPE_SERIAL => 'Serial Code',
            ]);
        
        $form->addInteger('price')->setLabel(___('Item Price'));
        $form->addInteger('qty')->setLabel(___('Available Quantity'));
        $form->addInteger('delivery')->setLabel(___('Delivery Timeframe, hours'));
        
        $form->addTextarea('instructions',
            ['class' => 'am-el-wide', 'rows' => 5])->setLabel(___('Instructions by seller'));
        
        $form->addTextarea('serial_codes',
            ['class' => 'am-el-wide', 'rows' => 5])->setLabel(___('Serial Codes'));
        
        foreach ($this->getModule()->categoryTypeTable()->findBy([]) as $type) {
            $form->addSelect('_category_' . $type->type_id,
                ['class' => 'am-combobox'])->setLabel($type->title)->loadOptions($this->getModule()->categoryTable()->getSelectOptions($type->pk()));
        }
        
        
        $form->addAdvCheckbox('is_approved')->setLabel(___('Is Approved?'));
        
        
        return $form;
        
    }
    
}

class Am_Grid_Filter_Listings extends Am_Grid_Filter_Marketplace
{
    protected function applyFilter()
    {
        class_exists('Am_Form', true);
        
        $vars = $this->vars ?? [];
        $ds = $this->grid->getDataSource();
        if (!empty($vars['dat1'])) {
            $ds->addWhere("t.created >=?", Am_Form_Element_Date::convertReadableToSQL($vars['dat1']));
        }
        if (!empty($vars['dat2'])) {
            $ds->addWhere("t.created <=?", Am_Form_Element_Date::convertReadableToSQL($vars['dat2']));
        }
        if (!empty($vars['status'])) {
            $ds->addWhere('t.status = ?', $vars['status']);
        }
        if (!empty($vars['text'])) {
            $text = Am_Di::getInstance()->db->escape("%" . $vars['text'] . "%");
            $ds->addWhere("t.title LIKE {$text} or t.description LIKE {$text} or t.instructions LIKE {$text}");
        }
        
    }
    
    function getStatusOptions($selected)
    {
        $statuses = [
            MarketplaceListing::STATUS_PUBLISHED,
            MarketplaceListing::STATUS_HOLD,
            MarketplaceListing::STATUS_SOLD,
            MarketplaceListing::STATUS_DRAFT
        ];
        $options = Am_Html::renderOptions(array_combine($statuses, array_map(function ($el)
        {
            return MarketplaceListingTable::getStatusText($el);
        }, $statuses)), $selected);
        return $options;
    }
}