<?php

class MarketplaceCategory extends Am_Record_WithData implements JsonSerializable
{
    function save()
    {
        $_ = parent::save(); // TODO: Change the autogenerated stub
        $this->getTable()->buildParents();
        return $_;
    }
    
    public function jsonSerialize()
    {
        $ret = $this->toArray();
        if($this->image){
            $ret['imageUrl'] = $this->getDi()->surl('upload/get/id/'.$this->image);
        }
        return $ret;
    }
}

class MarketplaceCategoryTable extends Am_Table_WithData
{
    
    protected $_table = '?_marketplace_category';
    protected $_key = 'category_id';
    protected $_recordClass = 'MarketplaceCategory';
    
    function buildParents()
    {
        $this->_db->query("delete from ?_marketplace_category_path");
        $tree = [];
        foreach ($this->selectObjects("select * from ?_marketplace_category order by parent_id") as $record) {
            if (empty($record->parent_id)) {
                $tree[$record->category_id] = [['id' => $record->category_id, 'name' => $record->name]];
            } else {
                $tree[$record->category_id] = array_merge($tree[$record->parent_id],
                    [['id' => $record->category_id, 'name' => $record->name]]);
            }
        }
        $insert = [];
        foreach ($tree as $category_id => $value) {
            $insert[] = sprintf("(%s, %s)", $category_id, $this->_db->escape(json_encode($value)));
        }
        $this->_db->query("insert into ?_marketplace_category_path (category_id, path) values " . implode(",",
                $insert));
    }
    
    function getChildren($category)
    {
        $total = 0;
        $ret = [];
        foreach ((array)$category as $category_id) {
            foreach ($this->getDi()->db->selectPage($total, "select * from ?_marketplace_category_path") as $row) {
                $path = json_decode($row['path'], true);
                if (count(array_filter($path, function ($el) use ($category_id)
                {
                    return $el['id'] == $category_id;
                }))) {
                    $ret[] = $row['category_id'];
                }
            }
        }
        return $ret;
    }
    
    function getSelectOptions($type_id = null)
    {
        $options = [];
        foreach ($this->getDi()->db->selectPage($total, "
                select p.category_id, p.path
                from ?_marketplace_category_path p left join ?_marketplace_category_sort s on p.category_id = s.category_id
                left join ?_marketplace_category c on p.category_id = c.category_id
                where 1=1 {and c.type_id=?}
                order by (s.sort+1)", $type_id ?? DBSIMPLE_SKIP) as $row) {
            $path = json_decode($row['path'], true);
            
            $options[$row['category_id']] = implode(" / ", array_map(function ($item)
            {
                return $item['name'];
            }, $path));
        }
        return $options;
    }
    
    function getListByType($type_id = null)
    {
        $list = $this->getDi()->marketplaceCategoryTable->selectObjects("
            select c.*, s.sort
            from ?_marketplace_category c left join ?_marketplace_category_sort s
            on c.category_id = s.category_id
            where 1=1 {and c.type_id=?d}
            ", $type_id ?: DBSIMPLE_SKIP);
        
        return $list;
        
    }
}