<?php

/**
 * @property int $message_id
 * @property int $transaction_id
 * @property int $user_id
 * @property string $name
 * @property string $type
 * @property string $message
 * @property string $read_by
 * @property string $date
 * @property string $remote_addr
 * @property string $user_agent
 */


class MarketplaceTransactionChat extends Am_Record implements JsonSerializable
{
    use Am_Marketplace_GetModule;
    
    const
        TYPE_BUYER = 'buyer',
        TYPE_SELLER = 'seller',
        TYPE_SYSTEM = 'system',
        TYPE_ADMIN = 'admin';
    
    
    public function jsonSerialize()
    {
        $ret = $this->toArray();
        
        unset($ret['remote_addr']);
        unset($ret['user_agent']);
        $ret['read_by'] = empty($ret['read_by']) ? [] : json_decode($ret['read_by'], true);
        $ret['date'] = amDatetime($ret['date']);
        return $ret;
    }
    
    function insert($reload = true)
    {
        if (!$this->transaction_id) {
            throw new Am_Exception_InternalError("transaction_id can't be empty");
        }
        
        parent::insert($reload);
    }
}

class MarketplaceTransactionChatTable extends Am_Table
{
    protected $_table = '?_marketplace_transaction_chat';
    protected $_key = 'message_id';
    protected $transaction_id;
    /**
     * @var MarketplaceTransaction $_transaction ;
     */
    protected $_transaction;
    
    function setTransaction(MarketplaceTransaction $transaction): MarketplaceTransactionChatTable
    {
        $this->_transaction = $transaction;
        
        $this->transaction_id = $transaction->pk();
        return $this;
    }
    
    function getTransaction(): ?MarketplaceTransaction
    {
        if (!empty($this->_transaction)) {
            return $this->_transaction;
        }
        
        if (!empty($this->transaction_id)) {
            $this->_transaction = $this->getDi()->marketplaceTransactionTable->load($this->transaction_id);
            return $this->_transaction;
        }
        return null;
        
    }
    
    function createRecord(array $row = null): MarketplaceTransactionChat
    {
        $record = parent::createRecord($row); // TODO: Change the autogenerated stub
        $record->transaction_id = $this->transaction_id;
        return $record;
    }
    
    function postUserMessage(
        User $user,
        string $message,
        string $remote_addr = null,
        string $user_agent = null
    ): MarketplaceTransactionChat {
        
        $record = $this->createRecord();
        $transaction = $this->getTransaction();
        
        if ($transaction->getSeller()->pk() == $user->pk()) {
            $record->type = MarketplaceTransactionChat::TYPE_SELLER;
        } else {
            $record->type = MarketplaceTransactionChat::TYPE_BUYER;
        }
        
        $record->user_id = $user->pk();
        $record->name = $user->login;
        $record->message = $message;
        $record->read_by = json_encode([$record->user_id]);
        $record->remote_addr = $remote_addr;
        $record->user_agent = $user_agent;
        $record->date = $this->getDi()->sqlDateTime;
        $record->insert();
        return $record;
    }
    
    function postAdminMessage(
        Admin $admin,
        string $message,
        string $remote_addr = null,
        string $user_agent = null
    ): MarketplaceTransactionChat {
        $record = $this->createRecord();
        $transaction = $this->getTransaction();
        
        $record->type = MarketplaceTransactionChat::TYPE_ADMIN;
        
        $record->user_id = $admin->pk();
        $record->name = ___('Admin : %s', $admin->login);
        $record->message = $message;
        $record->remote_addr = $remote_addr;
        $record->user_agent = $user_agent;
        $record->date = $this->getDi()->sqlDateTime;
        $record->insert();
        
        return $record;
    }
    
    
    function postSystemMessage($message)
    {
        $record = $this->createRecord();
        $record->type = MarketplaceTransactionChat::TYPE_SYSTEM;
        $record->name = ___('System Bot');
        $record->message = $message;
        $record->date = $this->getDi()->sqlDateTime;
        $record->insert();
        return $record;
    }
    
    function getLatestMessages(int $last_id = null, int $count = 20)
    {
        $params = [
            'transaction_id' => $this->_transaction->pk(),
        ];
        if (!empty($last_id)) {
            $params[] = ['message_id', '>', $last_id];
        }
        return array_reverse($this->findBy($params, 0, $count, 'message_id desc'));
    }
    
    function getTotal()
    {
        return $this->countBy(['transaction_id' => $this->_transaction->pk()]);
    }
    
    function getHistory(int $last_id, int $count = 20)
    {
        $params = [
            'transaction_id' => $this->_transaction->pk(),
        ];
        $params[] = ['message_id', '<', $last_id];
        return array_reverse($this->findBy($params, 0, $count, 'message_id desc'));
    }
    
    
}