<?php

namespace Am\Marketplace\Slim\Controller;

use Am\Marketplace\Slim\Controller;
use Am\Marketplace\Slim\Response;
use Am_Query;
use MarketplaceListing;
use Psr\Http\Message\ServerRequestInterface;

class GetListings extends Controller
{
    protected $sort = ['created-desc', 'created', 'price', 'price-desc', 'rating', 'delivery'];
    
    function buildQuery($user_id = null): Am_Query
    {
        
        $qry = new Am_Query($this->getModule()->listingsTable());
        $qry->clearFields();
        $qry->addField("DISTINCT t.*");
        
        $qry->leftJoin('?_user', 'u', 't.user_id = u.user_id');
        $qry->addField('marketplace_rating', 'rating');
        if (!empty($user_id)) {
            $qry->addWhere('t.user_id=?', $user_id);
            
        } else {
            $qry->addWhere('t.is_approved = 1');
            $qry->addWhere('t.status = ?', MarketplaceListing::STATUS_PUBLISHED);
        }
        return $qry;
    }
    
    function prepareData(array $results): array
    {
        return array_map(function (MarketplaceListing $listing)
        {
            $listing = $listing->jsonSerialize();
            unset($listing['instructions']);
            return $listing;
        }, $results);
        
    }
    
    function __invoke(ServerRequestInterface $request, Response $response, array $args): Response
    {
        $body = $request->getParsedBody();
        $qry = $this->buildQuery($request->getAttribute('type') ? $request->getAttribute('user')->pk() : null);
        if (!empty($body['filter']['text'])) {
            $qry->addWhere('t.title like ? or t.description like ?', $_ = "%" . $body['filter']['text'] . "%", $_);
        }
        
        if(!empty($body['filter']['category']))
        {
            foreach((array)$body['filter']['category'] as $type => $category)
            {
                $categories = $this->getModule()->categoryTable()->getChildren($category);
                if(!empty($categories))
                {
                    $qry->leftJoin("?_marketplace_listing_category", "cat{$type}", "t.listing_id = cat{$type}.listing_id");
                    $qry->addWhere("cat{$type}.category_id in (?a)", $categories);
                }
                
            }
            
        }
        if (isset($body['sort']) && in_array($body['sort'], $this->sort)) {
            $sortParts = explode("-", $body['sort']);
            $qry->setOrderRaw(implode(" ", $sortParts));
        }
        
        if (isset($body['id'])) {
            $qry->addWhere("t.listing_id=?", $body['id']);
        }
        $sql = $qry->getSql($body['skipFirst'] ?? null, $body['count'] ?? 20);
        $total = $this->getDi()->db->selectCell(sprintf("select count(*) from (%s) as tbl", $qry->getSql()));
        
        $results = $this->getModule()->listingsTable()->selectObjects($sql);
        if (!empty($body['id']) && !empty($results)) {
            /**
             * @var MarketplaceListing $listing
             */
            $listing = $results[0];
            $listing->recordView($request->getAttribute('user'));
        }
        
        return $response->withJson([
            'ok' => true,
            'list' => $this->prepareData($results),
            'total' => $total
        ]);
    }
}