<?php declare(strict_types=1);

namespace Am\Marketplace\Slim;

use Slim\Psr7\Response as SlimResponse;
use Slim\Psr7\Stream;

class Response extends SlimResponse
{
    public function withJson($data, $status = null, $encodingOptions = 0)
    {
        $response = $this->withBody(new Stream(fopen('php://temp', 'r+')));
        $response->body->write($json = json_encode($data, $encodingOptions));
        
        // Ensure that the json encoding passed successfully
        if ($json === false) {
            throw new RuntimeException(json_last_error_msg(), json_last_error());
        }
        
        $responseWithJson = $response->withHeader('Content-Type', 'application/json');
        if (isset($status)) {
            return $responseWithJson->withStatus($status);
        }
        
        return $responseWithJson;
    }
    
    
    public function withJsonError(
        string $errorMessage,
        int $statusCode = 403,
        int $errorCode = 1
    ): self {
        $response = new self;
        return $response->withJson([
            'ok' => false,
            'errorMessage' => $errorMessage,
            'errorCode' => $errorCode,
        ])->withStatus($statusCode);
    }
    
    static public function jsonError(
        string $errorMessage,
        int $statusCode = 403,
        int $errorCode = 1
    ) {
        return (new self)->withJsonError($errorMessage, $statusCode, $errorCode);
    }
    
    
    public function getJson()
    {
        $this->body->rewind();
        return json_decode($this->body->getContents(), true);
    }
    
}