<?php

class Am_Grid_Action_Group_SelfserviceCredit extends Am_Grid_Action_Group_Abstract
{
    protected $credit = false;
    protected $form;
    protected $_vars;

    public function __construct($credit = true)
    {
        $this->credit = (bool)$credit;
        parent::__construct(
            $credit ? "self-service-credit" : "self-service-debit",
            $credit ? ___("Self-Service Credit") : ___("Self-Service Debit")
        );
    }

    public function renderConfirmationForm($btn = null, $addHtml = null)
    {
        $this->getForm();
        $vars = $this->grid->getCompleteRequest()->toArray();
        $vars[$this->grid->getId() . '_confirm'] = 'yes';
        foreach ($vars as $k => $v)
            if ($this->form->getElementsByName($k))
                unset($vars[$k]);
        foreach(Am_Html::getArrayOfInputHiddens($vars) as $k => $v)
            $this->form->addHidden($k)->setValue($v);

        $url_yes = $this->grid->makeUrl(null);
        $this->form->setAction($url_yes);
        echo $this->renderTitle();
        echo (string)$this->form;
    }

    /**
     * @param int $id
     * @param User $record
     */
    public function handleRecord($id, $record)
    {
        switch ($this->credit) {
            case true:
                $this->grid->getDi()->selfserviceHistoryTable
                    ->credit($record, $this->_vars['value'], $this->_vars['type_id'], $this->_vars['description']);
                break;
            case false:
                $this->grid->getDi()->selfserviceHistoryTable
                    ->debit($record, $this->_vars['value'], $this->_vars['type_id'], $this->_vars['description']);
                break;
        }
    }

    protected function getForm()
    {
        if (!$this->form) {
            $this->form = new Am_Form_Admin('self-service');
            $id = $this->grid->getId();

            $options = $this->grid->getDi()->selfserviceTypeTable->getOptions();
            $type_id = $this->form->addSelect("{$id}_type_id")
                ->setLabel(___('Credit Type'))
                ->loadOptions($options);

            if (count($options) == 1) {
                reset($options);
                $type_id->setValue(key($options));
                $type_id->persistentFreeze(true);
                $type_id->toggleFrozen(true);
            }

            $value = $this->form->addInteger("{$id}_value", ['autofocus' => 'autofocus'])
                ->setLabel(___('Value'))
                ->setId('ss-value');

            $value->addRule('required');

            $this->form->addText("{$id}_description", ['class' => 'am-el-wide'])
                ->setLabel(___('Comment'));

            $this->form->addScript()
                ->setScript("jQuery('#ss-value').focus()[0].select()");

            $this->form->addSaveButton($this->credit ? ___('Credit') : ___('Debit'));
        }
        return $this->form;
    }

    public function run()
    {
        if (!$this->getForm()->validate()) {
            echo $this->renderConfirmationForm();
        } else {
            $prefix = $this->grid->getId().'_';
            foreach ($this->getForm()->getValue() as $k => $v) {
                if (strpos($k, $prefix)===0)
                    $this->_vars[substr($k, strlen($prefix))] = $v;
            }
            return parent::run();
        }
    }
}