<?php
/**
 * @table integration
 * @id pap
 * @title Post Affiliate Pro
 * @description An affiliate management script
 * @type Other..
 * @single_login 0
 * @different_groups 0
 * @visible_link https://www.postaffiliatepro.com
 * @am_protect_api 6.0
 */
class Am_Protect_Pap extends Am_Protect_Abstract{

    const PLUGIN_STATUS = self::STATUS_BETA;
    const PLUGIN_COMM = self::COMM_COMMERCIAL;
    const PLUGIN_REVISION = '6.3.21';

    const PAP = 'pap';
    const USER_NEED_SETPASS = 'user_need_setpass';
    const PAP_REFERRAL_ID = 'pap_ref_id';
    const PAP_PARENT_ID = 'pap_parent_id';
    const PAP_VISITOR_ID = 'pap_visitor_id';
    const DEFAULT_CAMPAIGN_ID = '11111111';
    protected  $session=null;

    public function init()
    {
        $this->getDi()->productTable->customFields()->add(new Am_CustomFieldText('pap_campaign_id', 'PAP Campaign ID','using to get list of commission groups
            and to set up commission group to user'));
        $this->getDi()->productTable->customFields()->add(new Am_CustomFieldText('pan_account', 'PAN Account ID', "Keep empty to use default setting"));
    }

    public function getPapSession()
    {
        if(is_null($this->session)) {
            $this->session = new Gpf_Api_Session($this->getConfig('server'));
            $this->session->login($this->getConfig('login'), $this->getConfig('password'));
        }
        return $this->session;
    }

    public function getPasswordFormat()
    {
        return self::PAP;
    }

    public function cryptPassword($pass, &$salt = null, User $user = null)
    {
        if(is_null($salt)){
            $salt = md5(uniqid());
        }
        $c = new Am_Crypt_Compat($salt);
        return $c->encrypt($pass);
    }

    public function includeAPI()
    {
        $path = Am_Di::getInstance()->root_dir . "/application/default/plugins/protect/pap/PapApi.class.php";
        if(!is_file($path) || !is_readable($path))
            return false;
        include_once($path);
        return true;
    }

    function getGroups(){
        $groups = [];
        try{
            $session = $this->getPapSession();
            $campaignids = [$this->getConfig('campaignid', self::DEFAULT_CAMPAIGN_ID)];
            foreach($this->getDi()->db->select("SELECT value from ?_data where `table` = 'product' and `key` = 'pap_campaign_id'") as $p)
                $campaignids[] = $p['value'];
            $campaignids = array_filter($campaignids);
            foreach($campaignids as $campaignid)
            {
                $request = new Gpf_Rpc_Request("Pap_Merchants_Campaign_CommissionGroups","loadCommissionGroups",$session);
                $request->addParam("campaignid",$campaignid);
                $request->sendNow();
                $response = $request->getStdResponse();
                foreach ($response as $responseItem)
                {
                    if ($responseItem[1] == 'name')
                        continue;
                    $groups[(string)$responseItem[0]] = (string)$responseItem[1].' [campaign ID:'.$campaignid.']';
                }
            }

        } catch (Exception $e) {
            $this->getDi()->logger->error("Exception", ["exception" => $e]);
        }
        return $groups;
    }

    public function isConfigured()
    {
        return $this->getConfig('server') &&
                $this->getConfig('login') &&
                $this->getConfig('password') &&
                $this->includeAPI();
    }

    function onSetupForms(Am_Event_SetupForms $event)
    {
        $form = new Am_Form_Setup('pap');
        $form->setTitle('PostAffiliate Pro');

        $fs = $form->addFieldset()->setLabel('API Authentication Information');
        $fs->addText('server', ['class' => 'am-el-wide'])
            ->setLabel("Server URL\n" .
                'example: http://example.com/pap/scripts/server.php');
        $fs->addText('login')->setLabel('Merchant Email');
        $fs->addText('password')->setLabel('Merchant Password');

        $fs->addText('campaignid')->setLabel("Campaign ID\n" .
            'using to get list of commission groups ' .
            'and to set up commission group to user');
        $form->setDefault('campaignid', self::DEFAULT_CAMPAIGN_ID);

        $fs = $form->addFieldset()->setLabel('Sale Tracking');
        $fs->addAdvCheckbox('sale_tracking')->setLabel('Enable Sale tracking through API');
        $fs->addText('sale_server', ['class' => 'am-el-wide'])
            ->setLabel("Sale Tracking Script URL\n" .
                'example: http://example.com/pap/scripts/sale.php');

        $fs->addText('pan_account', ['class' => 'am-el-wide'])
            ->setLabel("PAN Account ID\n" .
                'You can find it in the section Accounts>Account manager when you are logged in as a network owner. This setting applicable to PostAffiliateNetwork owners only. Leave it empty if you don\'t use PostAffiliateNetwork.');
        $form->addFieldsPrefix('protect.pap.');
        if ($plugin_readme = $this->getReadme()) {
            $plugin_readme = str_replace(
                ['%root_url%', '%root_surl%', '%root_dir%'], [ROOT_URL, ROOT_SURL, ROOT_DIR], $plugin_readme);
            $form->addEpilog('<div class="info"><pre>' . $plugin_readme . '</pre></div>');
        }
        $event->addForm($form);
    }


    function getPanAccountId(Invoice $invoice)
    {
        $products = $invoice->getProducts();
        return $products[0]->data()->get('pan_account')?:$this->getConfig('pan_account');
    }
    function onPaymentAfterInsert(Am_Event_PaymentAfterInsert $event)
    {
        if(!$this->getConfig('sale_tracking')) return;
        $payment = $event->getPayment();
        $invoice = $this->getDi()->invoiceTable->load($payment->invoice_id);
        try {
            $products = $invoice->getProducts();
            $saleTracker = new Pap_Api_SaleTracker($this->getConfig('sale_server'));
            if($acc = $this->getPanAccountId($invoice)) $saleTracker->setAccountId($acc);
            if($visitor_id = $invoice->getUser()->data()->get(self::PAP_VISITOR_ID)) $saleTracker->setVisitorId ($visitor_id);
            $sale = $saleTracker->createSale();
            $sale->setTotalCost($payment->amount);
            $sale->setOrderID($payment->receipt_id);
            $sale->setProductID($products[0]->pk());
            $sale->setData1($invoice->getUser()->email);
            $saleTracker->register();
        } catch (Exception $e) {
            $this->getDi()->logger->error("Exception", ["exception" => $e]);
        }
    }

    function onInvoicePaymentRefund(Am_Event $e){

        if(!$this->getConfig('sale_tracking')) return;

        $invoice = $e->getInvoice();
        $refund = $e->getRefund();

        if(!$refund->invoice_payment_id) return;

        $payment = $this->getDi()->invoicePaymentTable->load($refund->invoice_payment_id);
        try {
            $session = $this->getPapSession();
        } catch (Exception $e) {
            $this->getDi()->logger->error("Exception", ["exception" => $e]);
            return;
        }

        $request = new Pap_Api_TransactionsGrid($session);
        $request->addFilter("orderid", Gpf_Data_Filter::EQUALS, $payment->receipt_id);
        $request->setSorting('orderid', false);
        $request->addParam('columns', new Gpf_Rpc_Array([
            ['id'],
            ['transid'],
            ['campaignid'], ['orderid'], ['commission'], ['userid']
        ]));
        $request->sendNow();

        $grid = $request->getGrid();

        $recordset = $grid->getRecordset();

        foreach ($recordset as $rec) {
            $sale = new Pap_Api_Transaction($session);
            $sale->setTransid($rec->get('transid'));
            try {
                if (!$sale->load()) {
                    throw new Exception('Cannot load transaction for' . $rec->get('transid') . ' error: ' . $sale->getMessage());
                } else {
                    if($refund->record_type == InvoiceRefund::CHARGEBACK) $sale->chargeBack('ChargeBack received');
                    else $sale->refund('Refund received');
                }
            } catch(Exception $e) {
                $this->getDi()->logger->error("Exception", ["exception" => $e]);
                continue;
            }
         }
    }

    function getIntegrationSettingDescription(array $config)
    {
        $groups = array_combine((array) $config['group'], (array) $config['group']);
        try {
            foreach ($this->getGroups() as $id => $title) {
                if (!empty($groups[$id]))
                    $groups[$id] = '[' . $title . ']';
            }
        } catch (Am_Exception_PluginDb $e) {

        }
        return "Assign Group " . implode(",", array_values($groups));
    }

    function getIntegrationFormElements(HTML_QuickForm2_Container $group)
    {
        parent::getIntegrationFormElements($group);
        $groups = $this->getGroups();
        $group->addSelect('group', [], ['options'=>$groups])
            ->setLabel("Add user to PAP\n" .
                'Users subscribed to this product will be added to PAP as affiliates with selected commission group');

    }

    /** @return IntegrationTable */
    public function getIntegrationTable()
    {
        if (!$this->_integrationTable)
            $this->_integrationTable = $this->getDi()->integrationTable;
        return $this->_integrationTable;
    }

    /**
     * Check whether user should be added to pap or not.
     * @param User $user
     * @return bool
     */
    function isAccessEnabled(User $user=null)
    {
        if ($user && $user->pk()) {
            foreach ($this->getIntegrationTable()->getAllowedResources($user, $this->getId()) as $integration) {
                $vars = unserialize($integration->vars);
            }
        }
        return !empty($vars['group']) ? $vars['group'] : false;
    }

    function getPass(User $user){
        $pass = $this->getDi()->savedPassTable->findSaved($user, $this->getPasswordFormat());
        if(is_null($pass)) return md5(uniqid());
        $c = new Am_Crypt_Compat($pass->salt);
        return $c->decrypt($pass->pass);
    }

    function onSubscriptionChanged(Am_Event_SubscriptionChanged $event, User $oldUser = null)
    {
        if ($oldUser === null) $oldUser = $event->getUser();
        $added = $event->getAdded();
        $user = $event->getUser();
        $access = $this->isAccessEnabled($user);
        $pass = $this->getPass($user);

        try {
            $session = $this->getPapSession();
        } catch (Exception $e) {
            $this->getDi()->logger->error("Exception", ["exception" => $e]);
            return;
        }
        $affiliate = new Pap_Api_Affiliate($session);

        if($ref_id  = $user->data()->get(self::PAP_REFERRAL_ID)){
            // We have refferal ID try to load affiliate.
            $affiliate->setRefid($ref_id);
            try {
                $affiliate->load();
                $found = true;
                $create=0;
            } catch (Exception $e) {
                $found=false;

            }
        }

        if(!$found) {
            $affiliate = new Pap_Api_Affiliate($session);

            $affiliate->setUsername($oldUser->getEmail());

            $create = 0;

            try {
                $affiliate->load();
            } catch (Exception $e) {
                if(!$access) return;
                $create = 1;
            }
        }

        $status = ($access) ? 'A' : 'P';

        if(!$affiliate->getRefid()) $affiliate->setRefid($this->getReferralID($user, $session));
        $affiliate->setUsername($user->getEmail());
        $affiliate->setPassword($pass);
        $affiliate->setFirstname($user->name_f);
        $affiliate->setLastname($user->name_l);
        $affiliate->setStatus($status);
        $affiliate->setParentUserId($user->data()->get(self::PAP_PARENT_ID));
        if ($create) {
            if(!$affiliate->add())
                $this->getDi ()->logger->error('PAP Plugin Error:'.$affiliate->getMessage());
        } else {
            $affiliate->save();
        }

        //set up commission group
        if((bool)$access){
            try {
                //Pap_Features_Common_AffiliateGroupForm
                $request = new Gpf_Rpc_FormRequest("Pap_Features_CommissionGroups_AffiliateGroupForm", "add", $session);
                $request->setField('Id','');
                $request->setField('userid',$affiliate->getField('Id'));
                $request->setField('rstatus','A');
                $request->setField('commissiongroupid',$access);
                $campaign_id = 0;
                if(!empty($added))
                {
                    foreach($added as $product_id)
                    {
                        $product = $this->getDi()->productTable->load($product_id);
                        if($cid = $product->data()->get('pap_campaign_id'))
                            $campaign_id = $cid;
                    }
                }
                if($campaign_id)
                    $request->setField('campaignid',$campaign_id);
                else
                    $request->setField('campaignid',$this->getConfig('campaignid', self::DEFAULT_CAMPAIGN_ID));
                $request->sendNow();
            } catch(Exception $e) {
                $this->getDi()->logger->error("Exception", ["exception" => $e]);
            }
        }
    }

    function getReferralID(User $user, Gpf_Api_Session $session){
        $ref_id = $user->data()->get(self::PAP_REFERRAL_ID);
        if(!$ref_id){
        // Set to login name.
            $ref_id = $user->login;
            if($this->refExists($ref_id, $session)|| $this->userExists($ref_id, $session)){
                do{
                    $ref_id = substr(md5(uniqid()), 3,8);
                }while($this->refExists($ref_id, $session) || $this->userExists($ref_id, $session));
            }
            $user->data()->set(self::PAP_REFERRAL_ID, $ref_id)->update();
        }
        return $ref_id;
    }

    function refExists($ref_id, Gpf_Api_Session $session){
        $affiliate = new Pap_Api_Affiliate($session);
        $affiliate->setRefid($ref_id);
        try {
            if ($affiliate->load()) {
                return true;
            }
        } catch(Exception $e) {

        }
        return false;
    }

    function userExists($user_id, Gpf_Api_Session $session){
        $affiliate = new Pap_Api_Affiliate($session);
        $affiliate->setUserid($user_id);
        try {
            if ($affiliate->load()) {
                return true;
            }
        } catch(Exception $e) {

        }
        return false;
    }

    function onUserAfterUpdate(Am_Event_UserAfterUpdate $event)
    {
        $e = new Am_Event_SubscriptionChanged($event->getUser(), [], []);
        return $this->onSubscriptionChanged($e, $event->getOldUser());
    }

    function onSetPassword(Am_Event_SetPassword $event)
    {
        $user = $event->getUser();
        $user->data()->set(self::USER_NEED_SETPASS, null)->update();
        $e = new Am_Event_SubscriptionChanged($user, [], []);
        return $this->onSubscriptionChanged($e, $e->getUser());

    }

    function onUserAfterDelete(Am_Event_UserAfterDelete $event)
    {
		$user = $event->getUser();
        if($this->isAccessEnabled($user)) return;
        try{
            $session = $this->getPapSession();
        }catch (Exception $e){
            $this->getDi()->logger->error("Exception", ["exception" => $e]);
            return;
        }

        $affiliate = new Pap_Api_Affiliate($session);
        $affiliate->setUsername($user->getEmail());

        try {
            if ($affiliate->load()) {
                $affiliate->setStatus('D');
                $affiliate->save();
            }
        } catch(Exception $e) {

        }
    }

    function onCheckUniqEmail(Am_Event_CheckUniqEmail $event)
    {
        try {
            $session = $this->getPapSession();
            $affiliate = new Pap_Api_Affiliate($session);

            $affiliate->setUsername($event->getEmail());

            if ($affiliate->load()){
                if($user_id = $event->getUserId()){
                    if($this->getDi()->userTable->load($user_id)->data()->get(self::PAP_REFERRAL_ID) != $affiliate->getRefid())
                        $event->setFailureAndStop();
                }else
                    $event->setFailureAndStop();
            }
        } catch(Exception $e) {

        }
    }

    function onCheckUniqLogin(Am_Event_CheckUniqLogin $event)
    {
        try {
            $session = $this->getPapSession();
            $affiliate = new Pap_Api_Affiliate($session);
            if($this->userExists($event->getLogin(),$session) || $this->refExists($event->getLogin(), $session))
                $event->setFailureAndStop();
        } catch(Exception $e) {

        }
    }

    public function setPapFields(Am_Event $event)
    {
        $vars = $event->getVars();
        $user = $event->getUser();
        foreach([self::PAP_REFERRAL_ID, self::PAP_PARENT_ID] as $k){
            if(array_key_exists($k, $vars) && $vars[$k] && !$user->data()->get($k))
                $user->data()->set($k, $vars[$k]);
        }

        // Set pap visitor_id as well;
        if($visitor_id  = @$_COOKIE['PAPVisitorId'])
            $user->data()->set(self::PAP_VISITOR_ID, $visitor_id);

        $user->data()->update();
    }

    public function onSignupUserAdded(Am_Event $event)
    {
        $this->setPapFields($event);
    }

    public function onSignupUserUpdated(Am_Event $event)
    {
        $this->setPapFields($event);
    }

    public function onRebuild(Am_Event_Rebuild $event)
    {
        $batch = new Am_BatchProcessor([$this, 'batchProcess']);

        $context = $event->getDoneString();
        $this->_batchStoreId = 'rebuild-' . $this->getId() . '-' . $this->getDi()->session->getId();
        if ($batch->run($context)) {
            $event->setDone();
            $this->getDi()->store->delete($this->_batchStoreId);
        } else {
            $event->setDoneString($context);
        }
    }
    /*

    public function onGetMemberLinks(Am_Event $e)
    {
        try {
            $session = new Gpf_Api_Session($this->getConfig('server'));
            @$session->login($user->email, $pass, Gpf_Api_Session::AFFILIATE);
            $affiliate = new Pap_Api_Affiliate($session);
            $affiliate->setUsername(Am_Di::getInstance()->auth->getUsername()); //load by username
            $affiliate->load();
            $e->addReturn('affiliate login', 'http://EXAMPLE.com/affiliates/login.php?authToken='.$affiliate->getField('authtoken'));
        } catch (Exception $e) {
            $this->getDi()->logger->error("Exception", ["exception" => $e]);
        }

    }
    */

    public function onAuthAfterLogin(Am_Event_AuthAfterLogin $event)
    {
        if ($pass = $event->getPassword()) {
            try {
                $user = $event->getUser();
                $session = new Gpf_Api_Session($this->getConfig('server'));
                @$session->login($user->email, $pass, Gpf_Api_Session::AFFILIATE);
                if($sessId = $session->getSessionId()){
                    Am_Cookie::set('A_pap_sid', $sessId);
                    Am_Cookie::set('RPC_pap_sid', $sessId);
                }
            } catch (Exception $e) {
                $this->getDi()->logger->error("Exception", ["exception" => $e]);
            }
        }
    }

    public function batchProcess(&$context, Am_BatchProcessor $batch)
    {
        @list($step, $start) = explode('-', $context);
        $pageCount = 30;
        $q = new Am_Query(Am_Di::getInstance()->userTable);
        $count = 0;
        $records = $q->selectPageRecords($start / $pageCount, $pageCount);
        foreach ($records as $user) {
            $count++;
            /* @var $user User */
            $this->onSubscriptionChanged(new Am_Event_SubscriptionChanged($user, [], []));
        }
        if (!$count) {
            $context = null;
            return true;
        }
        $start += $count;
        $context = "0-$start";
    }


    function validate($var, HTML_QuickForm2_Element $el, Am_Form_Brick $brick){
        $session = $this->getPapSession();
        switch($el->getName()){
            case self::PAP_PARENT_ID :
                $parent = new Pap_Api_Affiliate($session);

                $parent->setUserid($var);

                try {
                    $parent->load();
                } catch(Exception $e) {
                    return  $brick->___("Incorrect Parent Affiliate ID");
                }
                break;
            case self::PAP_REFERRAL_ID :
                $affiliate = new Pap_Api_Affiliate($session);
                $affiliate->setRefid($var);

                try {
                    if ($affiliate->load()) {
                        return $brick->___("Such affiliate ID exist");
                    }
                } catch(Exception $e) {

                }
                break;
        }
    }

    function onLoadBricks()
    {
        require_once dirname(__FILE__) . '/pap-bricks.php';
    }
}