<?php
/**
 * @table integration
 * @id vbulletin
 * @title vBulletin
 * @visible_link http://www.vbulletin.org/
 * @hidden_link http://www.vbulletin.org/
 * @description vBulletin is a powerful, scalable and fully customizable
 * forums package for your web site. It has been written using the Web's
 * quickest-growing scripting language; PHP, and is complemented with a
 * highly efficient and ultra fast back-end database engine
 * built using MySQL.
 *
 * vBulletin is the ideal community solution for all medium-to-large sites.
 * @different_groups 1
 * @single_login 1
 * @type Bulletin Boards
 * @am_protect_api 6.0
 */
class Am_Protect_Vbulletin extends Am_Protect_Databased
{
    const PLUGIN_STATUS = self::STATUS_PRODUCTION; //dev status
    const PLUGIN_COMM = self::COMM_COMMERCIAL; //paid
    const PLUGIN_REVISION = '6.3.21';

    protected $groupMode = self::GROUP_MULTI;
    protected $guessTablePattern = "user";
    protected $guessFieldsPattern = [
        'userid',
        'usergroupid',
        'username',
        'styleid'
    ];

    const VBULLETIN_5_1 = 'vbulletin_5_1';
    const VBULLETIN_5_2 = 'vbulletin_5_2';

    protected function isvBulletinPluginEnabled()
    {
        if($this->isConfigured())
        {
            try
            {
                $this->getDb()->query("select amemberid from ?_user limit 1");
                return true;

            }
            catch(Am_Exception_Db $e)
            {
                return false;
            }
        }

    }
    function parseExternalConfig($path)
    {
        $config_path = [
            $path. "/includes/config.php",
                             $path . "/core/includes/config.php"
        ];
        foreach ($config_path as $p)
        {
            if (file_exists($p))
            {
                include($p);
                break;
            }
        }
        if(!is_array($config))
            throw new Am_Exception_InputError("Specified path is not a valid vBulletin installation!");

        // Try to fetch license info from vBulletin config;
        $files = [
            $path . "/global.php",
            $path . "/core/global.php",
        ];
        $vblicense = null;
        foreach ($files as $p) {
            $file = @file_get_contents($p);
            if (preg_match('/Licence Number (\S+)/',$file, $regs)) {
                $vblicense = $regs[1];
                break;
            }
        }
        return [
            'db' => $config['Database']['dbname'],
            'prefix' => $config['Database']['tableprefix'],
            'user' => $config['MasterServer']['username'],
            'host' => $config['MasterServer']['servername'],
            'pass' => $config['MasterServer']['password'],
            'vblicense' => $vblicense,
            'cookie_prefix' => $config['Misc']['cookieprefix']
        ];
    }

    public function afterAddConfigItems($form)
    {
        parent::afterAddConfigItems($form);
        $form->addText("protect.{$this->getId()}.vblicense")
            ->setLabel("Your vB License Key\n" .
            "necessary to make single-login feature working. Looks like: E5291yp9");
        $form->addText("protect.{$this->getId()}.user_title")
            ->setLabel("Default User Title\n" .
                'aMember will set User Title based on user group. ' .
                'If group does not have user_title set, this one will be used');
        $form->addText("protect.{$this->getId()}.cookie_prefix")
          ->setLabel($this->getTitle()." Cookie Prefix\n" .
            "Prefix that all vBulletin cookies will have");
        $form->addSelect("protect.{$this->getId()}.version", [], ["options" => ['5.7' => '5.7', '5.2' => '5.2', '5.1' =>'5.1', 5=>5, 4=>4, 3=>3]])
             ->setLabel("vBulletin version");
    }

    public function getIntegrationFormElements(HTML_QuickForm2_Container $group)
    {
        parent::getIntegrationFormElements($group);
        $options = [];
        foreach($this->getAvailableUserGroups() as $gr){
            $options[$gr->getId()] = $gr->getTitle();
        }

        $group->addMagicSelect('additional_groups', [], ['options'=>$options])->setLabel('Additional Groups');
    }

    public function getCookiePrefix()
    {
        // Doesn't used for 5.1
        if($this->getConfig('version') ==5.1) return "";

        return ($this->getConfig('cookie_prefix') ? $this->getConfig('cookie_prefix') : 'bb').($this->getConfig('version')==4 ? '_' : '');
    }

    public function getLoggedInRecord()
    {
        $hash = filterId(@$_COOKIE[$this->getCookiePrefix().'sessionhash']);
        if (empty($hash)) return;
        $r = $this->getDb()->selectRow("SELECT
            u.*
            FROM ?_session s RIGHT JOIN ?_user u ON
                s.userid = u.userid
            WHERE sessionhash=?", $hash);
        if ($r)
            return $this->getTable()->createRecord($r);
    }

    public function logoutUser(User $user)
    {
        $pref = $this->getCookiePrefix();
        $hash = filterId(@$_COOKIE[$this->getCookiePrefix().'sessionhash']);
        if(!empty($hash)){
            //delete session from database;
            $this->getDb()->query("delete from ?_session where sessionhash=?",$hash);
        }
        foreach (['sessionhash','userid','password'] as $k)
            Am_Cookie::set($pref.$k, null, time() - 3600*24*1000, '/', $this->getDi()->request->getHttpHost());
    }

    protected function getIdHash()
    {
        if ($this->getConfig('version') >= 5)
        {
            //These are set from the web page bot not from CLI
            $ip = @$_SERVER['REMOTE_ADDR'];
            if (isset($_SERVER['HTTP_CLIENT_IP']))
            {
                $ip = $_SERVER['HTTP_CLIENT_IP'];
            }
            else if (isset($_SERVER['HTTP_X_FORWARDED_FOR'])
                && preg_match_all('#\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}#s', $_SERVER['HTTP_X_FORWARDED_FOR'], $matches))
            {
                // try to avoid using an internal IP address, its probably a proxy
                $ranges = [
                    [ip2long('10.0.0.0'), ip2long('10.255.255.255')],
                    [ip2long('127.0.0.0'), ip2long('127.255.255.255')],
                    [ip2long('169.254.0.0'), ip2long('169.254.255.255')],
                    [ip2long('172.16.0.0'), ip2long('172.31.255.255')],
                    [ip2long('192.168.0.0'), ip2long('192.168.255.255')],
                ];
                foreach ($matches[0] AS $xip)
                {
                    $ip_long = ip2long($xip);
                    if ($ip_long === false) continue;
                    $private_ip = false;
                    foreach ($ranges AS $range)
                        if ($ip_long >= $range[0] AND $ip_long <= $range[1])
                        {
                            $private_ip = true;
                            break;
                        }
                    if (!$private_ip)
                    {
                        $ip = $xip;
                        break;
                    }
                }
            }
            else if (isset($_SERVER['HTTP_FROM']))
            {
                $alt_ip = $_SERVER['HTTP_FROM'];
            }


            $length = 1; // default value  $this->datastore->get_value('options')['ipcheck'];
            $ip = implode('.', array_slice(explode('.', $ip), 0, 4 - $length));
            return md5($_SERVER['HTTP_USER_AGENT'] . $ip);
        } else {
            return md5($s=$_SERVER['HTTP_USER_AGENT'] . implode('.', array_slice(explode('.', $_SERVER['REMOTE_ADDR']), 0, 3)));
        }
    }

    public function loginUser(Am_Record $record,$password)
    {
        $hash = filterId(@$_COOKIE[$this->getCookiePrefix().'sessionhash']);
        if(empty($hash)){
            // Cookie is not set create new one;
            do{
                $hash = md5(uniqid());
            }while($this->getDb()->selectCell("select count(*) from ?_session where sessionhash =?",$hash));

            Am_Cookie::set($this->getCookiePrefix().'sessionhash', $hash, 0, '/', $this->getDi()->request->getHttpHost());
        }
        if($session = $this->getDb()->selectRow('select * from ?_session where sessionhash =?', $hash)){
            $this->getDb()->query("update ?_session set
                                    userid=?, host=?, idhash=?, lastactivity=?, useragent=?, styleid=?, languageid=?, loggedin=? where sessionhash=?",
                                    $record->userid, $_SERVER['REMOTE_ADDR'], $this->getIdHash(), time(), $_SERVER['HTTP_USER_AGENT'], $record->styleid,
                                    $record->languageid, 2, $hash
                    );
        }else{
            $this->getDb()->query("insert into ?_session set
                                    sessionhash=?, userid=?, host=?, idhash=?, lastactivity=?, useragent=?, styleid=?, languageid=?, loggedin=?",
                                    $hash, $record->userid, $_SERVER['REMOTE_ADDR'], $this->getIdHash(), time(), $_SERVER['HTTP_USER_AGENT'], $record->styleid,
                                    $record->languageid, 2
                    );
        }

    }

    public function getAvailableUserGroupsSql()
    {
        return "SELECT
            usergroupid as id,
            title as title,
            (usergroupid=8) as is_banned,
            (usergroupid IN (5,6,7)) as is_admin
            FROM ?_usergroup";
    }

    public function cryptPassword($pass, & $salt = null, User $user = null)
    {
        if($this->getConfig('version') >= 5.2)
        {
            return password_hash(md5($pass), PASSWORD_DEFAULT);
        }
        else return parent::cryptPassword ($pass, $salt, $user);
    }

    public function getPasswordFormat()
    {
        if($this->getConfig('version') >= 5.2)
            return self::VBULLETIN_5_2;
        else
            return ($this->getConfig('version') >= 5.1 ? SavedPassTable::PASSWORD_PASSWORD_HASH : SavedPassTable::PASSWORD_MD5_MD5_PASS_SALT);
    }

    public function createTable()
    {
        $table = new Am_Protect_Vbulletin_Table($this, $this->getDb(), '?_user', 'userid');
        $tz = $this->_db->selectCell("select value from ?_setting where varname='timeoffset'");
        $threadedmode = 1;
        $autosubscribe = -1;
        $showvbcode = 1;
        $separate_options = [
            'threadedmode' => [
                'thread_linear_oldest' => 0,
                'thread_linear_oldest' => 3,
                'thread_threaded' => 1,
                'thread_hybrid' => 2
            ],
            'autosubscribe' => [
                'subscribe_none' => -1,
                'subscribe_nonotify' => 0,
                'subscribe_instant' => 1,
                'subscribe_daily' => 2,
                'subscribe_weekly' => 3
            ],
            'showvbcode' => [
                'vbcode_none' => 1,
                'vbcode_standard' => 2,
                'vbcode_enhanced' => 3
            ]
        ];
        $opts = unserialize($this->_db->selectCell("select data from ?_datastore  where title='bitfields'"));
        if(isset($opts['misc']['useroptions']) && isset($opts['misc']['regoptions'])){
            $defaultregoptions = $this->_db->selectCell("select value from ?_setting where varname='defaultregoptions'");
            $regoptions = 0;
            foreach([
                'invisible'         => 'invisiblemode',
                'receivepm'         => 'enablepm',
                'emailonpm'         => 'emailonpm',
                'showreputation'    => 'showreputation',
                'showvcard'         => 'vcard',
                'showsignatures'    => 'signature',
                'showavatars'       => 'avatar',
                'showimages'        => 'image',
                'vm_enable'         => 'vm_enable',
                'vm_contactonly'    => 'vm_contactonly',
                'pmdefaultsavecopy' => 'pmdefaultsavecopy',
                'adminemail' => 'adminemail',
                    ] as $optionname => $bitfield){
                    if (($defaultregoptions & @$opts['misc']['regoptions'][$bitfield]) === @$opts['misc']['regoptions'][$bitfield])
                    {
                        $regoptions+=$opts['misc']['useroptions'][$optionname];
                    }
                }
            foreach($separate_options as $optname => $vals)
            {
                foreach($vals as $k => $v)
                {
                    if (($defaultregoptions & $opts['misc']['regoptions'][$k]) === $opts['misc']['regoptions'][$k])
                        $$optname = $v;
                }
            }
        }
        else $regoptions = '7255';
        $tz = $tz ? $tz : 0;
        $fieldsMapping = [
                [Am_Protect_Table::FIELD_LOGIN, 'username'],
                [Am_Protect_Table::FIELD_EMAIL, 'email'],
                [':0', 'styleid'],
                [':1','showvbcode'],
                [Am_Protect_Table::FIELD_ADDED_STAMP,'joindate'],
                [':-1','daysprune'],
                [':0','lastvisit'],
                [':1','startofweek'],
                [':1','reputationlevelid'],
                [':'.$tz, 'timezoneoffset'],
                [':-1', 'referrerid'],
                [':1', 'languageid'],
                [':'.$this->getDi()->sqlDate, 'passworddate'],
                [':'.$regoptions, 'options'],
                [Am_Protect_Table::FIELD_REMOTE_ADDR, 'ipaddress'],
        ];
        if($this->getConfig('version') >= 5.7)
            $fieldsMapping[] = [Am_Protect_Table::FIELD_LOGIN, 'displayname'];
        foreach(array_keys($separate_options) as $optname)
            $fieldsMapping[] = [':'.$$optname, $optname];

        if($this->getConfig('version')>=5.1)
        {
            $fieldsMapping[] = [Am_Protect_Table::FIELD_PASS, 'token'];
        }else{
            $fieldsMapping[] = [Am_Protect_Table::FIELD_PASS, 'password'];
            $fieldsMapping[] = [Am_Protect_Table::FIELD_SALT, 'salt'];
        }
        if($this->isvBulletinPluginEnabled()){
            $fieldsMapping[] = [Am_Protect_Table::FIELD_ID, 'amemberid'];
        }

        $table->setFieldsMapping($fieldsMapping);
        return $table;

    }

    function chooseGroups($groups, User $user = null)
    {
        $ret = [];
        $primary = [];
        $additional = [];
        foreach ($groups as $config){
            $primary[] = $config['gr'];
            if(is_array($config['additional_groups'])){
                $additional = array_merge($additional, $config['additional_groups']);
            }
        }
        rsort($primary);
        $groups = array_unique(array_filter(array_merge($primary, $additional)));
        $groups = $this->addSuperGroups($groups, $user);

        if ($this->_priority === null) $this->_initPriority();
        usort($groups, [$this, '_compareGroups']);
        return $groups;
    }


    function canAutoCreate()
    {
        return true;
    }

    function canAutoCreateFromGroups()
    {
        return true;
    }
}

class Am_Protect_Vbulletin_Table extends Am_Protect_Table
{
	function updateCounts()
	{
        $members = $this->getPlugin()->getDb()->selectRow("
            SELECT SUM(IF(lastvisit >= ?, 1, 0)) AS active,
			COUNT(*) AS users, MAX(userid)
			AS max FROM ?_user
            ", time() - 30 * 86400);


         $newuser = $this->getPlugin()->getDb()->selectRow("SELECT userid, username FROM ?_user WHERE userid=?", $members['max']);

		// make a little array with the data
		$values = [
			'numbermembers' => $members['users'],
			'activemembers' => $members['active'],
			'newusername' => $newuser['username'],
			'newuserid' => $newuser['userid']
        ];
		// update the special template
		$this->getPlugin()->getDb()->query("REPLACE INTO ?_datastore (title, data, unserialize) VALUES ('userstats', ?, 1)", serialize($values));
	}

    public function insertFromAmember(User $user, SavedPass $pass, $groups)
    {
        $record = parent::insertFromAmember($user, $pass, $groups);
        $this->updatePassword($record, $pass);
        $this->_db->query("INSERT IGNORE INTO ?_usertextfield (userid) VALUES (?)", $record->pk());
        $this->_db->query("INSERT IGNORE INTO ?_userfield (userid) VALUES (?)", $record->pk());
        $this->updateCounts();
    }

    public function getUserTitle($groupid)
    {
        $title = $this->_db->selectCell("SELECT usertitle FROM ?_usergroup WHERE usergroupid = ?d", $groupid);
        if(!$title) $title = $this->getPlugin()->getConfig('user_title');
        return $title;
    }

    public function getGroups(Am_Record $record)
    {
        $groups = [$record->usergroupid];
        $groups = array_merge($groups, explode(',', $record->membergroupids));
        return array_filter($groups);
    }

    public function setGroups(Am_Record $record, $groups)
    {
        $record->usergroupid = $record->displaygroupid = array_shift($groups);
        $record->membergroupids = implode(',', $groups);
        $record->usertitle = $this->getUserTitle($record->usergroupid);
        $record->updateSelectedFields(['usergroupid', 'membergroupids', 'displaygroupid', 'usertitle']);
    }

    public function updatePassword(Am_Record $record, SavedPass $saved)
    {
        if($this->getPlugin()->getConfig('version')>=5.1){
            $hash_info = password_get_info($saved->pass);
            if($hash_info['algo']!== PASSWORD_BCRYPT)
            {
                $oldSaved = $this->getDi()->savedPassTable->findSaved($this->findAmember($record), SavedPassTable::PASSWORD_MD5_MD5_PASS_SALT);
                if($oldSaved)
                {
                    $saved->pass = $oldSaved->pass." ".$oldSaved->salt;
                    $record->updateQuick('scheme', 'legacy');
                }
            }
            else
            {
                $record->updateQuick('scheme', 'blowfish:'.@$hash_info['options']['cost']);
            }
        }

        parent::updatePassword($record, $saved);
        $record->updateQuick('passworddate', $this->getDi()->sqlDate);
    }

    public function updateFromAmember(Am_Record $record, User $user, $groups) {
        parent::updateFromAmember($record, $user, $groups);
        $record->amemberid = $user->pk();
        $record->updateSelectedFields(['amemberid']);
    }

    public function remove()
    {
        throw new Am_Exception_NotImplemented();
    }

    function checkPassword(Am_Record $record, User $user, $plaintextPass = null)
    {
        if ($plaintextPass && $this->getPlugin()->getConfig('version') >= 5.2)
        {
            if (password_verify(md5($plaintextPass), $record->token))
                return true;
        }
        return parent::checkPassword($record, $user, $plaintextPass);
    }

}
