import React, {FC, useEffect} from "react";
import {Grid, useTheme} from "@mui/material";
import {TChatWindowProps} from "../types";
import ChatMessage from "./Message";

const InfiniteScroll = React.lazy(() => import("react-infinite-scroller"));

const ChatWindow: FC<TChatWindowProps> = (props) => {
    const theme = useTheme();

    let windowRef: HTMLDivElement | null;


    useEffect(() => {
        if (windowRef)
            windowRef.scrollTop = windowRef.scrollHeight ?? 200;
    }, [props.messages]);

    return <Grid container direction="column" sx={{
        height: "200px",
        maxHeight: "200px",
        border: `1px solid ${theme.palette.secondary.light}`,
        overflow: "auto"
    }} wrap={"nowrap"} ref={(node) => windowRef = node} gap={1}>
        <InfiniteScroll
            loadMore={() => props.onNeedMore()}
            hasMore={props.total > props.messages.length}
            isReverse={true}
            threshold={50}
        >
            {props.messages && props.messages.map((el, i) => {
                return <ChatMessage message={el} isMine={el.user_id === props.ownerId} isSystem={el.type == 'system'}
                                    key={i}/>
            })}
        </InfiniteScroll>

    </Grid>
}

export default ChatWindow;