import React, {FC, useState} from "react"
import {Backdrop, Box, Grid, useTheme} from "@mui/material";
import {TListingsGridProps} from "../types";
import ListingBox from "./Listing/Box";
import noop from "../../Utils/noop";
import {LoadingBox} from "../../Utils/Loading";
import {grey} from "@mui/material/colors";

const InfiniteScroll = React.lazy(() => import("react-infinite-scroller"));

const ListingsGrid: FC<TListingsGridProps> = (props) => {
    const isLoading = props.isLoading ?? false;
    const showCategory = props.showCategory ?? true;
    const showOwner = props.showOwner ?? true;
    const showOrder = props.showOrder ?? true;
    const showStatus = props.showStatus ?? true;
    const canEdit = props.canEdit ?? false;
    const canDelete = props.canDelete ?? false;
    const onDelete = props.onDelete ?? noop;
    const onNeedMoreRecords = props.onNeedMoreRecords ?? noop;

    const [showFilter, toggleFilter] = useState(false);
    const theme = useTheme();

    return (
        <Box sx={{position: 'relative', minHeight: '200px'}}>

            <Backdrop
                sx={{color: '#fff', zIndex: (theme) => theme.zIndex.drawer + 1, position: 'absolute'}}
                open={isLoading}
                invisible={true}
            >
                <LoadingBox size={2}/>
            </Backdrop>
            <InfiniteScroll
                loadMore={() => onNeedMoreRecords()}
                hasMore={props.hasMoreRecords}
                loader={<LoadingBox size={2} key={0}/>}
            >
                <Grid container spacing={2} justifyContent={"flex-start"}>

                    {props.records.map((r, i) => <Grid item xs={12} key={i}
                                                       sx={{backgroundColor: i % 2 == 0 ? grey[50] : "none"}}>
                        <ListingBox
                            record={r}
                            showOwner={showOwner}
                            showCategory={showCategory}
                            showOrder={showOrder}
                            showStatus={showStatus}
                            canEdit={canEdit && (r.status.active || r.status.draft)}
                            canDelete={canDelete}
                            onDelete={onDelete}
                        />
                    </Grid>)}

                </Grid>

            </InfiniteScroll>
        </Box>
    )
}
export default ListingsGrid;
