import React, {FC} from "react";
import {TTransactionSellerSidebarProps} from "../../types";
import {Grid, styled} from "@mui/material";
import TransactionInfo from "./Info";
import Header from "./Header";
import ChatWidget from "../../../Chat/ChatWidget";
import {useMarketplaceStore} from "../../Store";
import TransactionActions from "./Actions";
import TransactionSerialCodes from "./SerialCodes";

const Description = styled("pre")({
    whiteSpace: "pre-wrap"
});

const TransactionSellerSidebar: FC<TTransactionSellerSidebarProps> = (props) => {
    const rootStore = useMarketplaceStore();


    return <Grid item container xs={6} sx={{paddingLeft: "1em", paddingRight: "1em"}} alignItems={"flex-start"}
                 flexDirection={"column"} columnSpacing={2}>
        <TransactionInfo transaction={props.transaction}/>

        <TransactionSerialCodes
            show={props.transaction.serial_codes != undefined && props.transaction.status == 'completed'}
            codes={props.transaction.serial_codes ?? []}/>
        {props.transaction.status == 'pending' &&
            <Grid item container direction={"column"}>
                <Header>Actions</Header>
                <Grid item sx={{paddingTop: "0.5em"}}>
                    <TransactionActions onTransactionRevert={props.onTransactionRevert}
                                        onConfirmDelivery={props.onConfirmDelivery}/>
                </Grid>
            </Grid>
        }
        <Grid item container direction={"column"}>
            <Header>Chat With Buyer</Header>
            <Grid item sx={{paddingTop: "0.5em"}}>
                <ChatWidget chatId={props.transaction.transaction_id} ownerId={props.transaction.seller_id}
                            apiUrl={rootStore.apiUrl('/transaction/chat')}/>
            </Grid>

        </Grid>


    </Grid>

}
export default TransactionSellerSidebar;