import React, {useState} from "react"
import {Grid, Typography} from "@mui/material";
import PageHeader from "../Layout/PageHeader";
import {observer} from "mobx-react";
import {useCategoryStore} from "../../AdminCategory/Store";
import {useMarketplaceStore} from "../Store";
import {useNavigate} from "react-router-dom";
import {TListingFormError, TListingFormValue} from "../types";
import MarketplaceLayout from "../Layout";
import {TApiSuccessResponse} from "../../types";

const ListingForm = React.lazy(() => import("../Layout/Listing/Form"))


const AddListing = observer(() => {
    const categoryStore = useCategoryStore();
    const [formError, setFormError] = useState<TListingFormError | undefined>(undefined);
    const store = useMarketplaceStore();
    const navigate = useNavigate();
    const [isFormLoading, toggleFormLoading] = useState(false);
    const onSubmit = (formValue: TListingFormValue) => {
        toggleFormLoading(true);
        const upload = formValue.upload ? store.apiUpload(formValue.upload) : Promise.resolve({
            ok: true,
            status: 'ok',
            error: ""
        } as unknown as TApiSuccessResponse);

        upload.then((response) => {
            if ('upload_id' in response)
                formValue.image = response.upload_id as string;

            return response.ok ? store.apiRequest('/listing/save', {form: formValue}) : response

        }).then((response) => {
            if ('status' in response) {
                if (response.status == 'error') {
                    return Promise.reject(response.error);
                }
            }
            navigate('/my');
        }).catch((error) => {
            setFormError(error);
        }).finally(() => {
            toggleFormLoading(false);
        })

    }

    return (
        <MarketplaceLayout>
            {store.userStore.user && <Grid container direction={"row-reverse"} spacing={2}>
                <Grid item xs={12}>
                    <PageHeader>Add Listing</PageHeader>
                </Grid>
                <Grid item xs={12} lg={4}>
                    <Typography component={"div"} variant={"body1"} color="secondary" fontWeight={"bold"}>Safe
                        Deal</Typography>
                    <Typography component={"div"} variant={"body1"} color="secondary" gutterBottom>
                        Order process implements "Safe Deal" scheme.
                        You will receive credits only after item delivery confirmation.
                    </Typography>
                    <Typography component={"div"} variant={"body1"} color="secondary" gutterBottom>
                        Here is what happens when user makes purchase:
                        <ul>
                            <li>Item will be removed from listing</li>
                            <li>Credits will be substructed from user's balance</li>
                            <li>User will receive instructuions specified in item. In instructions you need to provide
                                information about item delivery,
                                or your contact details also you can ask user to provide contact info back
                            </li>
                            <li>Once item is received, user confirms order and you get credits</li>
                        </ul>
                    </Typography>

                </Grid>
                <Grid item xs={12} lg={8}>

                    <ListingForm
                        categoryOptions={categoryStore.categorySelectOptions}
                        user={store.userStore.user}
                        onCancel={() => navigate('/my', {replace: true})}
                        onSubmit={onSubmit}
                        isLoading={isFormLoading}
                        error={formError}
                    />
                </Grid>

            </Grid>
            }
        </MarketplaceLayout>

    )
})
export default AddListing;