import {TCategory, TCategorySelectOption} from "../AdminCategory/types";
import {ReactNode} from "react";
import {ListingRecord} from "./Store/Listing";

export type TMarketplaceStoreConfig = {
    amUrl: string
    selfUrl: string
}

export interface TListingFormValue {
    title: string
    description?: string
    qty?: number
    category_id?: Record<string, string>
    price: number
    instructions?: string
    status: string
    upload?: File
    image?: string
    imageUrl?: string
    delivery?: number
    listing_type?: string
    serial_codes?: string
}

export type TCategoryItem = {
    id: number
    name: string
}
export type TCategoryChain = Array<TCategoryItem>

export interface TListing extends TListingFormValue {
    created: string
    listing_id: string
    is_approved: boolean
    categories?: Array<TCategoryChain>
    views: number
    delivery?: number
    owner: {
        user_id: number
        username: string
        rating?: number
        total?: number
    }
}

export interface TTransactionListing extends TListingPublicData {
    instructions: string
}


export interface TListingOwnerData {
    owner: {
        username: string,
        totalSold: number,
        rating: number
    }
}

export interface TListingPublicData extends TListingOwnerData {
    title: string
    image: string | null
    created: string
    listingTypeText: string
    categories: Array<TCategoryChain>
    description: string

}


export type TListingFormProps = {
    listing?: TListingFormValue
    listing_id?: number
    categoryOptions: Array<TCategorySelectOption>
    onCancel?: () => void
    onSubmit?: (formValue: TListingFormValue) => void
    error?: TListingFormError
    isLoading?: boolean
    user: TCurrentUser
    isDisabled?: boolean
}

export type TCurrentUser = {
    user_id: string
    login: string
    email: string
    name_f: string
    name_l: string
    canPostListings: boolean
    canSetQty: boolean
    balance: number
    historyUpdates: Record<"sales" | "orders", Array<number>>
    hasOrders: boolean
    hasSales: boolean
}

export type TNavigationTab = {
    regex: RegExp
    to: string
    label?: ReactNode
    icon?: JSX.Element
    order: number
    value: number
    component?: JSX.Element
}

export type TListingFormError = { [key: keyof TListingFormValue]: string } | string;

export type TListingsGridProps = {
    records: Array<ListingRecord>
    isLoading?: boolean
    showOwner?: boolean
    showCategory?: boolean
    showOrder?: boolean
    showStatus?: boolean
    canEdit?: boolean
    canDelete?: boolean
    onDelete?: (id: string) => void
    onNeedMoreRecords?: () => void
    hasMoreRecords: boolean
}

export type TSearchBarProps = {
    filterIcon: true
    showFilter: boolean
    onFilterClick: () => void
    onSearchChange: (search: string) => void
    search?: string
} | {
    filterIcon: false
    onSearchChange: (search: string) => void
    search?: string
}

export type TMarketplaceLayoutProps = {
    filter?: ReactNode
}

export type TImageUploadPros = {
    onSetFile: (file: File) => void
    onDeleteFile: () => void
    url?: string
    maxUploadSize: number
    maxUploadWidth: number
}

export type TModuleConfig = {
    maxUploadWidth: number,
    maxUploadSize: number,
    buyCreditsUrl: string
}

export type TListingQtyButtonProps = {
    max: number
    onChange: (qty: number) => void
    initial: number
}

export type TPurchaseListingData = {
    listing_id: number
    qty: number
}

export type TOrderSidebarProps = {
    listing: ListingRecord
    currentUser?: TCurrentUser
    onPurchaseListing: (data: TPurchaseListingData) => Promise<any>
    buyCreditsUrl: string
}

export type TTransaction = {
    transaction_id: number
    buyer_id: number
    seller_id: number
    buyer: string
    started: string
    updated: string
    finished: string
    qty: number
    total: number
    status: string
    statusText: string
    price: number
    comment: string
    expire: string
    serial_codes?: Array<string>
    listing: TTransactionListing
}

export type TTransactionSidebarProps = {
    transaction: TTransaction
    onCommentUpdate: (comment: string) => Promise
    onSaleConfirming: () => Promise
}
export type TTransactionSellerSidebarProps = {
    transaction: TTransaction
    onConfirmDelivery: () => Promise
    onTransactionRevert: () => Promise

}

export type TTransactionSerialCodesProps = {
    show: boolean,
    codes: Array<string>
}

export type TTransactionSellerSidebarActionProps = {
    onConfirmDelivery: () => Promise
    onTransactionRevert: () => Promise
}


export type TReceiptProps = {
    price: ReactNode
    qty: ReactNode
    total: ReactNode
    qtyTitle: ReactNode
    delivery: ReactNode
    deliveryTitle: ReactNode
}

export type TConfirmDialogProps = {
    open: boolean
    onClose: () => void
    onConfirmed: () => void
    title: string
    okBtnText?: string
    cancelBtnText?: string
}

export type TCategoryType = {
    type_id: string
    slug: string
    title: string
    description: string
    sort_order: number
    list: Array<TCategory>
}

export type TFilter = {
    category?: Record<string, string>
    text?: string
}

export type TApiGetListingsOptions = {
    all: boolean
    skipFirst?: number
    filter?: TFilter
}



