<?php declare(strict_types=1);

namespace Am\Marketplace\Slim\Controller;

use Am\Marketplace\Slim\Controller;
use Am\Marketplace\Slim\Response;
use Am_Exception_Security;
use MarketplaceListing;
use Psr\Http\Message\ServerRequestInterface;

class SaveListing extends Controller
{
    
    
    function validateForm(array $form)
    {
        $_ = [];
        foreach (['title', 'price', 'instructions', 'listing_type'] as $key) {
            if (empty($form[$key])) {
                $_[$key] = 'Required';
            }
        }
        if (!empty($_)) {
            return $_;
        }
        
        if (intval($form['price']) <= 0) {
            $_['price'] = ___('Use only digits');
        }
        
        if ($form['listing_type'] == MarketplaceListing::TYPE_REGULAR && intval($form['delivery']) <= 0) {
            $_['delivery'] = ___('Required. Use only digits');
        }
        
        if (!empty($form['qty']) && $form['qty'] != 1 && !$this->getModule()->isMarketplaceQtyEnabled($this->getDi()->auth->getUser())) {
            return ___("You can't set listing quantity");
        }
        
        
        if (!in_array($form['status'], [MarketplaceListing::STATUS_PUBLISHED, MarketplaceListing::STATUS_DRAFT])) {
            $_['status'] = ___("Listing status can't be changed");
        }
        
        if (empty($form['category_id'])) {
            return ___('Please select  category for listing');
        }
        
        if ($form['listing_type'] == MarketplaceListing::TYPE_SERIAL && empty($form['serial_codes'])) {
            $_['serial_codes'] = ___('Please specify at least one serial code');
        }
        
        return $_;
    }
    
    function __invoke(ServerRequestInterface $request, Response $response, array $args): Response
    {
        $body = $request->getParsedBody();
        $form = $body['form'] ?? null;
        $listing_id = $body['listing_id'] ?? null;
        if (empty($form)) {
            return $this->errorResponse(___('Please complete form below and click save'));
        }
        
        $errors = $this->validateForm($form);
        
        if (!empty($errors)) {
            return $this->errorResponse($errors);
        }
        
        /**
         * @var MarketplaceListing $listing
         */
        if (!is_null($listing_id)) {
            $listing = $this->getDi()->marketplaceListingTable->load($listing_id);
            if ($listing->user_id != $this->getDi()->auth->getUserId()) {
                throw new Am_Exception_Security("You can't edit this listing");
            }
            
            if (
                !in_array($listing->status, [MarketplaceListing::STATUS_DRAFT, MarketplaceListing::STATUS_PUBLISHED]) ||
                !in_array($form['status'], [MarketplaceListing::STATUS_DRAFT, MarketplaceListing::STATUS_PUBLISHED])
            ) {
                return $this->errorResponse(___("You can't edit this listing"));
            }
            
            $listing->status = $form['status'];
            
        } else {
            $listing = $this->getDi()->marketplaceListingRecord;
            $listing->user_id = $this->getDi()->auth->getUserId();
            $listing->status = $form['status'];
            $listing->created = $this->getDi()->sqlDateTime;
        }
        
        $listing->listing_type = $form['listing_type'];
        switch ($listing->listing_type) {
            case MarketplaceListing::TYPE_SERIAL :
                $serials = array_unique(array_filter(array_map(function ($el)
                {
                    return trim($el);
                }, explode("\n", $form['serial_codes']))));
                $listing->serial_codes = implode("\n", $serials);
                $listing->qty = count($serials);
                break;
            default:
                $listing->delivery = $form['delivery'];
                $listing->qty = $form['qty'] ?? 1;
        }
        $listing->title = $form['title'];
        $listing->description = $form['description'];
        $listing->instructions = $form['instructions'];
        $listing->price = $form['price'];
        $listing->image = isset($form['image']) && (intval($form['image']) > 0) ? $form['image'] : null;
        $listing->is_approved = 0;
        $listing->save();
        $listing->setCategories($form['category_id']);
        
        $this->getModule()->newListingNotificationToAdmin($listing);
        
        return $response->withJson([
            'ok' => true,
            'status' => 'ok',
            'listing' => $listing->toArray()
        ]);
    }
}
