<?php declare(strict_types=1);

namespace Am\Marketplace\Slim\Middleware;

use Am\Marketplace\Slim\Response;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;

class RequireOwnListingID implements MiddlewareInterface
{
    
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $body = $request->getParsedBody();
        
        if (empty($body['listing_id'])) {
            return Response::jsonError(___("Listing ID is empty in request"));
        }
        
        $listing = $request->getAttribute('container')->getModule()->listingsTable()->load($body['listing_id']);
        if (empty($listing)) {
            return Response::jsonError(___("Unable to find listing"));
        }
        if ($request->getAttribute('user')->user_id != $listing->user_id) {
            return Response::jsonError(___("Listing was created by another user!"));
        }
        
        return $handler->handle($request->withAttribute('listing', $listing));
        
    }
}