<?php

class Marketplace_AdminCategoriesController extends Am_Mvc_Controller_Grid
{
    use Am_Marketplace_GetModule;
    function checkAdminPermissions(Admin $admin)
    {
        return $admin->hasPermission(Bootstrap_Marketplace::ADMIN_PERM) || $admin->isSuper();
    }
    
    function preDispatch()
    {
        $this->getView()->headLink()->appendStylesheet([
            'href' => "https://fonts.googleapis.com/css?family=Roboto:300,400,500,700&display=swap",
            'rel' => 'stylesheet',
            'media' => 'screen',
            'extras' => ['crossorigin' => 'anonymous']
        ]);
        $this->getView()->headLink()->appendStylesheet([
            'href' => "https://fonts.googleapis.com/icon?family=Material+Icons",
            'rel' => 'stylesheet',
            'media' => 'screen',
            'extras' => ['crossorigin' => 'anonymous']
        ]);
        parent::preDispatch(); // TODO: Change the autogenerated stub
    }
    
    function editAction()
    {
        $categoryType = $this->getDi()->marketplaceCategoryTypeTable->load($this->getParam('type_id'));
        $this->getView()->assign('categoryType', $categoryType);
        $this->getView()->display('admin/marketplace/categories.phtml');
        
    }
    
    function createGrid()
    {
        $ds = new Am_Query($this->getModule()->categoryTypeTable());
        $ds->addOrderRaw('sort_order+0');
        $grid = new Am_Grid_Editable('_category_type',___('Category Types'), $ds, $this->getRequest(), $this->getView());
    
        $grid->addField('type_id', ___('ID'));
        $grid->addField('title', ___('Title'));
        $grid->addField('slug', ___('Slug'));
        
        $grid->addField('_categories', ___('Categories'), false)->setRenderFunction(function($record){
            $count = $this->getModule()->categoryTable()->countBy(['type_id' => $record->pk()]);
            return sprintf(
                "<td><a href='%s' target='_top'>%s</a></td>",
                $this->getDi()->surl('marketplace/admin-categories/edit', ['type_id' => $record->type_id]),
                ___('Edit Categories (%s)', $count)
            );
        });
        
        $grid->setForm(function(){
            $form = new Am_Form_Admin();
            $form->addText('title', ['class' => 'am-el-wide'])->setLabel(___("Category Type Title"));
            $form->addText('slug')->setLabel(___('Type Slug'));
            $form->addTextarea('description', ['rows' => '5', 'class'=>'am-el-wide'])->setLabel('Description');
            return $form;
        });
        $grid->actionAdd(new Am_Grid_Action_Sort_Types());
        return $grid;
    }
}
class Am_Grid_Action_Sort_Types extends Am_Grid_Action_Sort_Abstract
{
    protected function setSortBetween($item, $after, $before)
    {
        $this->_simpleSort($this->grid->getDi()->marketplaceCategoryTypeTable, $item, $after, $before);
    }
}
