<?php

class Marketplace_AdminTransactionsController extends Am_Mvc_Controller_Grid
{
    function checkAdminPermissions(Admin $admin)
    {
        return $admin->hasPermission(Bootstrap_Marketplace::ADMIN_PERM);
    }
    
    function preDispatch()
    {
        parent::preDispatch(); // TODO: Change the autogenerated stub
        if ($this->getParam('user_id')) {
            $this->layout = "admin/user-layout.phtml";
        }
    }
    
    function createGrid()
    {
        $user_id = $this->getParam("user_id");
        
        $ds = new Am_Query($this->getModule()->transactionTable());
        
        $ds->leftJoin('?_user', 'b', 'b.user_id = t.buyer_id');
        $ds->leftJoin('?_user', 's', 's.user_id = t.seller_id');
        
        $ds->addField('b.login', 'buyer');
        $ds->addField('s.login', 'seller');
        $ds->addField('b.user_id', 'buyer_id');
        $ds->addField('s.user_id', 'seller_id');
        $ds->addField('transaction_id', '_log');
        
        $ds->addOrderRaw('t.updated desc');
        if (!empty($user_id)) {
            $ds->addWhere("b.user_id=? or s.user_id=?", $user_id, $user_id);
        }
        
        
        $grid = new Am_Grid_ReadOnly('_marketplace_transactions', ___('Transactions'), $ds, $this->getRequest(),
            $this->getView());
        
        $grid->addField('transaction_id', ___('ID'));
        $grid->addField('updated', ___('Updated'));
        $grid->addField('title', ___('Item'));
        $grid->addField('status', ___('Status'))->setRenderFunction(function ($record)
        {
            $color = "black";
            switch ($record->status) {
                case 'pending' :
                    $color = 'yellow';
                    break;
                case 'expired' :
                case 'reverted' :
                    $color = 'red';
                    break;
                case 'completed' :
                    $color = 'green';
                    break;
                case 'sent' :
                    $color = 'blue';
                    break;
            }
            return sprintf("<td><span style='color: %s'>%s</td>", $color,
                MarketplaceTransaction::statusText($record->status));
        });
        $linkTo = function ($who)
        {
            return function ($record) use ($who)
            {
                $login = $record->{$who};
                $id = $record->{$who . "_id"};
                
                return sprintf("<td><a href='%s' target='_top'>%s</a></td>",
                    $this->getDi()->surl("admin-users?_u_a=edit&_u_id={$id}"), $login);
            };
        };
        $grid->addField('buyer', ___('Customer'))->setRenderFunction($linkTo('buyer'));
        $grid->addField('seller', ___('Seller'))->setRenderFunction($linkTo('seller'));
        
        
        $grid->addField((new Am_Grid_Field_Expandable('_log', ___('Log'),
            false))->setAjax($this->getDi()->surl('marketplace/admin-transactions/ajax/?id={transaction_id}'))->setPlaceholder(___('Details')));
        $grid->setFilter(new Am_Grid_Filter_Transactions());
        return $grid;
        
    }
    
    function ajaxAction()
    {
        $id = $this->getRequest()->getParam('id');
        $ret = [];
        foreach ($this->getModule()->logTable()->selectObjects("
            select l.*, u.login as `user` from ?_marketplace_transaction_log l left join ?_user u on l.user_id = u.user_id where transaction_id = ? order by `date`
            ", $id) as $log) {
            $ret[] = sprintf("<p>%s: %s  <br/>user: %s, IP: %s, UserAgent: %s</p>", $log->date, $log->record,
                $log->user, $log->remote_addr, $log->user_agent);
        }
        print implode("", $ret);
    }
    
    /**
     *
     * @return Bootstrap_Marketplace $module
     */
    function getModule()
    {
        return parent::getModule();
    }
}

class Am_Grid_Filter_Transactions extends Am_Grid_Filter_Marketplace
{
    protected function applyFilter()
    {
        class_exists('Am_Form', true);
    
        $vars = $this->vars ?? [];
        $ds = $this->grid->getDataSource();
        if (!empty($vars['dat1'])) {
            $ds->addWhere("t.updated >=?", Am_Form_Element_Date::convertReadableToSQL($vars['dat1']));
        }
        if (!empty($vars['dat2'])) {
            $ds->addWhere("t.updated <=?", Am_Form_Element_Date::convertReadableToSQL($vars['dat2']));
        }
        if (!empty($vars['status'])) {
            $ds->addWhere('t.status = ?', $vars['status']);
        }
        if (!empty($vars['text'])) {
            $text = Am_Di::getInstance()->db->escape("%" . $vars['text'] . "%");
            $ds->addWhere("t.title LIKE {$text} or b.login LIKE {$text} or s.login LIKE {$text}");
        }
        
    }
    
    function getStatusOptions($selected)
    {
        $statuses = [
            MarketplaceTransaction::STATUS_REVERTED,
            MarketplaceTransaction::STATUS_SENT,
            MarketplaceTransaction::STATUS_COMPLETED,
            MarketplaceTransaction::STATUS_PENDING,
            MarketplaceTransaction::STATUS_EXPIRED
        ];
        $options = Am_Html::renderOptions(array_combine($statuses, array_map(function ($el)
        {
            return MarketplaceTransaction::statusText($el);
        }, $statuses)), $selected);
        return $options;
    }
}