<?php declare(strict_types=1);

namespace Am\Marketplace\Slim\Controller;

use Am\Marketplace\Slim\Controller;
use Am\Marketplace\Slim\Response;
use MarketplaceTransaction;
use Psr\Http\Message\ServerRequestInterface;

class TransactionComment extends Controller
{
    
    function __invoke(ServerRequestInterface $request, Response $response, array $args): Response
    {
        $params = $request->getParsedBody();
        
        if (empty($params['transaction_id']) || empty($params['comment'])) {
            $this->errorResponse(___('Incomplete request'));
        }
        
        /**
         * @var MarketplaceTransaction $transaction ;
         */
        $transaction = $this->getModule()->transactionTable()->load($params['transaction_id']);
        
        if (empty($transaction)) {
            $this->errorResponse(___('Unable to find transaction'));
        }
        
        if (($transaction->buyer_id != $request->getAttribute('user')->pk()) || strlen($transaction->comment) > 0) {
            $this->errorResponse(___('You are not allowed to change this transaction'));
        }
        
        $transaction->setComment($params['comment']);
        
        $this->getModule()->transactionStatusChangedNotification($transaction);
        
        return $response->withJson([
            'ok' => true,
            'status' => 'ok',
            'transaction' => $transaction
        ]);
    }
}