<?php declare(strict_types=1);

namespace Am\Marketplace\Slim\Controller;

use Am\Marketplace\Slim\Controller;
use Am\Marketplace\Slim\Response;
use Am_Query;
use MarketplaceTransaction;
use Psr\Http\Message\ServerRequestInterface;

class Transactions extends Controller
{
    
    
    function __invoke(ServerRequestInterface $request, Response $response, array $args): Response
    {
        $body = $request->getParsedBody();
        
        $qry = new Am_Query($this->getModule()->transactionTable());
        
        $type = $body['type'] ?? 'orders';
        
        if ($type == 'orders') {
            $qry->addWhere("t.buyer_id =?", $user_id = $request->getAttribute('user')->pk());
        } else {
            $qry->addWhere("t.seller_id =?", $user_id = $request->getAttribute('user')->pk());
        }
        if (!empty($body['filter'])) {
            $qry->leftJoin("?_user", 's', 's.user_id = t.seller_id');
            $qry->leftJoin("?_user", 'b', 'b.user_id = t.buyer_id');
            $qry->addWhere("s.login like ? or b.login like ? or t.title like ?", $_ = "%" . $body['filter'] . "%", $_,
                $_);
        }
        
        if (!empty($body['id'])) {
            $qry->addWhere('t.transaction_id=?', $body['id']);
        }
        $qry->addOrderRaw("t.updated desc, t.started desc");
        
        $sql = $qry->getSql($body['skipFirst'] ?? null, $body['count'] ?? 20);
        
        $results = $this->getModule()->transactionTable()->selectObjects($sql);
        
        $total = $this->getDi()->db->selectCell(sprintf("select count(*) from (%s) as tbl", $qry->getSql()));
        if (!empty($body['id']) && !empty($results)) {
            /**
             * @var MarketplaceTransaction $transaction
             */
            $transaction = $results[0];
            $transaction->recordTransactionUpdate($request->getAttribute('user'), true);
        }
        
        return $response->withJson([
            'ok' => true,
            'list' => $results,
            'total' => $total
        ]);
        
        
    }
}