<?php

class MarketplaceListing extends Am_Record_WithData implements JsonSerializable
{
    use Am_Marketplace_GetModule;
    const
        STATUS_PUBLISHED = 'published',
        STATUS_DRAFT = 'draft',
        STATUS_SOLD = 'sold',
        STATUS_HOLD = 'hold';
    
    
    public $status;
    public $listing_id;
    public $user_id;
    public $_categories = null;
    
    
    function setCategories(array $categories): self
    {
        
        $this->getDi()->db->query("delete from ?_marketplace_listing_category where listing_id=?", $this->pk());
        
        $insert = [];
        foreach ($categories as $type_id=>$category) {
            $insert[] = sprintf("(%s, %s, %s)", $this->pk(), $this->getDi()->db->escape($category), $type_id);
        }
        if (!empty($insert)) {
            $this->getDi()->db->query("insert into ?_marketplace_listing_category (listing_id, category_id, type_id) values " . implode(", ",
                    $insert));
        }
        
        return $this;
    }
    
    function getCategoryPath(): array
    {
        if (is_null($this->_categories)) {
            $this->_categories = array_map(function ($el)
            {
                return json_decode($el, true);
            }, $this->getDi()->db->selectCol("
                select path
                from ?_marketplace_category_path left join ?_marketplace_listing_category using(category_id)
                where listing_id=?", $this->pk()));
        }
        return $this->_categories;
    }
    
    
    function insert($reload = true)
    {
        parent::insert($reload); // TODO: Change the autogenerated stub
        
    }
    
    function category_id()
    {
        $ret = [];
        foreach($this->getDi()->db->selectPage($total, "select category_id, type_id from ?_marketplace_listing_category where listing_id=?", $this->pk()) as $record)
        {
            $ret[$record['type_id']] = $record['category_id'];
        }
        return $ret;
    }
    public function jsonSerialize()
    {
        $ret = $this->toArray();
        $ret['created'] = amDate($ret['created']);
        $ret['categories'] = $this->getCategoryPath();
        
        $user = $this->getDi()->userTable->load($this->user_id);
        $ret['is_approved'] = (bool)$ret['is_approved'];
        if ($ret['image'] && preg_match("/^[0-9]+$/", $ret['image'])) {
            $ret['imageUrl'] = $this->getDi()->surl("upload/get/id/{$ret['image']}");
        }
        $ret['category_id'] = $this->category_id();
        
        $ret['owner'] = [
            'user_id' => $user->pk(),
            'username' => $user->login,
            'rating' => $user->marketplace_rating,
            'total' => $user->marketplace_transactions
        ];
        return $ret;
    }
    
    function getUser(): User
    {
        return $this->getDi()->userTable->load($this->user_id);
    }
    
    function pendingTransaction(MarketplaceTransaction $transaction)
    {
        $this->qty -= $transaction->qty;
        if ($this->qty <= 0) {
            $this->status = self::STATUS_HOLD;
        }
        $buyer = $transaction->getBuyer();
        /**
         * @var SelfserviceHistoryTable $tbl
         */
        $tbl = $this->getDi()->selfserviceHistoryTable;
        
        $tbl->debit($buyer, $transaction->total, 1, ___('Marketplace order# ' . $transaction->pk()),
            'marketplace_transaction', $transaction->pk());
        
        $this->update();
        
    }
    
    function recordView(User $user=null)
    {
        if (is_null($user) || ($this->user_id != $user->pk())) {
            $this->views++;
            $this->updateSelectedFields('views');
        }
    }
    
    function isPurchasePossible(User $user, $qty)
    {
        if ($this->user_id == $user->pk())
        {
            throw new Am_Exception_InputError(___("You can't purchase own item"));
        }
    
        if ($this->status !== self::STATUS_PUBLISHED || !$this->is_approved) {
            throw new Am_Exception_InputError("Listing is not available for purchase");
        }
    
        $owner = $this->getDi()->userTable->load($this->user_id);
    
        if ($qty > 1 && !$this->getModule()->isMarketplaceQtyEnabled($owner)) {
            throw new Am_Exception_InputError(___('You can purchase only one item at a time'));
        }
    
        if ($qty > $this->qty) {
            throw new Am_Exception_InputError(___('Requested quantity is too high'));
        }
        return true;
    
    }
    
}

class MarketplaceListingTable extends Am_Table_WithData
{
    
    protected $_table = '?_marketplace_listing';
    protected $_key = 'listing_id';
    protected $_recordClass = MarketplaceListing::class;
    
    static function getStatusText(string $text): string
    {
        switch ($text) {
            case MarketplaceListing::STATUS_PUBLISHED :
                return ___('Published');
            case MarketplaceListing::STATUS_DRAFT :
                return ___('Draft');
            case MarketplaceListing::STATUS_HOLD :
                return ___('Pending Purchase');
            case MarketplaceListing::STATUS_SOLD :
                return ___('Sold Out');
            default:
                return ___('Unknown');
        }
    }
}