import React, {FC, useState} from "react"
import {Button, Grid, TextField, useTheme} from "@mui/material";
import {TChatInputBoxProps} from "../types";
import Loading from "../../Utils/Loading";

const ChatInputBox: FC<TChatInputBoxProps> = (props) => {
    const [loading, setLoading] = useState(false);
    const [message, setMessage] = useState("");
    const theme = useTheme();
    const sendHandler = () => {
        setLoading(true);
        props.onMessagePost(message).then(() => {
            setMessage("")
        }).finally(() => {
            setLoading(false)
        });

    }
    return <Grid container direction={"row"}>
        <Grid item xs>
            <TextField
                size={"small"}
                fullWidth
                variant={"standard"}
                value={message}
                onChange={(event) => setMessage(event.target.value)}
                onKeyUp={(event) => (event.keyCode == 13) && sendHandler()}
            />
        </Grid>
        <Grid item>
            <Button disabled={loading} onClick={() => sendHandler()}>
                Send {loading && <Loading/>}
            </Button>
        </Grid>
    </Grid>
}
export default ChatInputBox;