import React, {FC} from "react"
import {TChatMessageProps} from "../types";
import {Box, Grid, Typography, useTheme} from "@mui/material";
import {grey, lightBlue, lightGreen} from "@mui/material/colors";

const ChatMessage: FC<TChatMessageProps> = (props) => {
    const theme = useTheme();
    const borderColor = props.isSystem ? grey[100] : (props.isMine ? lightGreen[100] : lightBlue[100])
    const backgroundColor = props.isSystem ? grey[50] : (props.isMine ? lightGreen[50] : lightBlue[50]);

    return <Grid container direction={props.isMine ? "row-reverse" : "row"}>
        <Box sx={{
            border: `1px solid ${borderColor}`,
            backgroundColor: backgroundColor,
            padding: "0.3em",
            margin: "0.3em",
            borderRadius: "3px"
        }}>
            <Grid container direction={"column"}>
                {!props.isSystem &&
                    <Grid item>
                        <Typography variant={"body2"}
                                    color={theme.palette.secondary.dark}>{props.isMine ? "" : props.message.name}</Typography>
                    </Grid>
                }
                <Grid item>
                    <Typography variant={"body1"}>{props.message.message}</Typography>
                </Grid>
                <Grid item alignSelf={"flex-end"} sx={{marginTop: "0.5em"}}>
                    <Typography variant={"body2"} fontSize={"10px"}
                                color={theme.palette.secondary.main}>{props.message.date}</Typography>
                </Grid>
            </Grid>
        </Box>
    </Grid>

}
export default ChatMessage;
