import React, {FC, useEffect} from "react";
import {TData} from "./types";
import {MarketplaceStore, MarketplaceStoreContext} from "./Marketplace/Store";
import {ThemeProvider} from "@mui/material";
import theme from "./Marketplace/theme";
import MarketplaceRoutes from "./Marketplace/Routes";

const Marketplace: FC<{ data: TData }> = (props) => {
    const rootStore = new MarketplaceStore({
        selfUrl: props.data.selfUrl ?? '',
        amUrl: props.data.amUrl ?? ''
    })
    let fetchUserTimer: NodeJS.Timer;
    useEffect(() => {
        rootStore.init();
        if (!fetchUserTimer) {
            fetchUserTimer = setInterval(() => {
                rootStore.userStore.fetchUser();
            }, 60000);
        }
    }, [props.data])

    return (
        <ThemeProvider theme={theme}>
            <MarketplaceStoreContext.Provider value={rootStore}>
                <MarketplaceRoutes selfUrl={props.data.selfUrl ?? ""}/>
            </MarketplaceStoreContext.Provider>
        </ThemeProvider>
    );
}
export default Marketplace;
