import React, {FC, ReactElement} from "react";
import {Grid, styled, Typography, useTheme} from "@mui/material";
import ListingImage from "./Image";
import OwnerInfo from "./OwnerInfo";
import CategoryList from "./CategoryList";
import {TListingPublicData} from "../../types";

const Description = styled("pre")({
    whiteSpace: "pre-wrap"
});


const ListingItem: FC<{ listing: TListingPublicData, sidebar?: ReactElement }> = ({listing, sidebar}) => {

    const theme = useTheme()
    return <>
        <Grid item>
            <Typography variant={"h5"} sx={{marginBottom: "1em"}}>{listing.title}</Typography>
        </Grid>
        <Grid item>
            <Grid container direction={"row"}>
                <Grid item container xs={6} sx={{borderRight: `1px solid ${theme.palette.secondary.light}`}}>
                    <Grid item container xs={4} sx={{
                        minWidth: "200px",
                        height: "200px",
                        alignItems: "center",
                        justifyContent: "center",
                        border: `1px solid ${theme.palette.secondary.light}`
                    }}>
                        <ListingImage src={listing?.image ?? null} maxWidth={"200px"}
                                      maxHeight={"200px"}></ListingImage>
                    </Grid>
                    <Grid item xs sx={{paddingLeft: "1em", paddingRight: "1em"}}>
                        <Typography color={"secondary"} variant={"body2"} component={"span"}>Seller: </Typography>
                        <OwnerInfo
                            record={listing}/>
                        <Typography variant={"body2"} color={theme.palette.secondary.main}>Posted
                            at {listing?.created}</Typography>
                        <Typography variant={"body2"} color={theme.palette.secondary.main}>Item
                            Type: {listing.listingTypeText}</Typography>
                        <CategoryList categories={listing?.categories}/>
                    </Grid>
                    <Grid item sx={{padding: "1em 0.5em"}} xs={12}>
                        <Description>{listing?.description}</Description>
                    </Grid>

                </Grid>
                {sidebar}
            </Grid>
        </Grid>
    </>
}
export default ListingItem