import {Box, IconButton, InputAdornment, TextField} from "@mui/material";
import React, {FC, FormEvent, useEffect, useState} from "react";
import SearchIcon from '@mui/icons-material/Search';
import {FilterAlt} from "@mui/icons-material";
import {TSearchBarProps} from "../types";
import DeleteOutlineIcon from '@mui/icons-material/DeleteOutline';

const SearchBar: FC<TSearchBarProps> = (props) => {
    const [search, setSearch] = useState("");
    useEffect(() => {
        setSearch(props.search ?? "");
    }, [props.search])
    return (

        <Box
            component="form"
            sx={{display: 'flex', alignItems: 'center'}}
            onSubmit={(event: FormEvent) => {
                event.preventDefault();
                props.onSearchChange(search)
            }}
        >
            <TextField size={"small"} fullWidth label="Search" id="fullWidth"
                       onChange={(event) => setSearch(event.target.value ?? "")}
                       value={search}
                       InputProps={search ? {
                           endAdornment: <InputAdornment position="end">
                               <IconButton
                                   aria-label="toggle password visibility"
                                   onClick={() => {
                                       setSearch("");
                                       props.onSearchChange("")
                                   }}
                                   edge="end"
                               >
                                   <DeleteOutlineIcon/>
                               </IconButton>
                           </InputAdornment>
                       } : {}}
            />
            <IconButton type="button" sx={{p: '10px'}} aria-label="search" onClick={() => props.onSearchChange(search)}>
                <SearchIcon/>
            </IconButton>
            {props.filterIcon &&
                <IconButton type="button" sx={{p: '10px'}} aria-label="search" onClick={() => props.onFilterClick()}>
                    <FilterAlt color={props.showFilter ? "primary" : "secondary"}/>
                </IconButton>}
        </Box>
    );
}
export default SearchBar;