import React, {FC, useState} from "react";
import {Alert, Button, Grid} from "@mui/material";
import Loading from "../../../Utils/Loading";
import ConfirmDialog from "../ConfirmDialog";
import {TTransactionSellerSidebarActionProps} from "../../types";


const TransactionActions: FC<TTransactionSellerSidebarActionProps> = (props) => {

    const [processingError, setProcessingError] = useState("");
    const [processingStatusUpdate, toggleProcessingStatusUpdate] = useState(false);
    const [statusUpdateDialog, toggleStatusUpdateDialog] = useState(false);
    const [revertError, setRevertError] = useState("");
    const [revertingTransaction, toggleRevertingTransaction] = useState(false);
    const [showRevertDialog, toggleRevertDialog] = useState(false);

    const statusUpdate = () => {
        toggleProcessingStatusUpdate(true)
        toggleStatusUpdateDialog(false)
        setProcessingError("")
        props.onConfirmDelivery().then(() => {

        }).catch((e: string) => {
            setProcessingError(e);
        }).finally(() => {
            toggleProcessingStatusUpdate(false);
        })
    }
    const revertTransaction = () => {
        toggleRevertingTransaction(true)
        toggleRevertDialog(false)
        setRevertError("")
        props.onTransactionRevert().then(() => {

        }).catch((e: string) => {
            setRevertError(e);
        }).finally(() => {
            toggleRevertingTransaction(false);
        })
    }

    return <>
        <Grid item container sx={{padding: '1em 0 1em 0'}} direction={"column"}>
            {processingError &&
                <Grid item sx={{marginBottom: "0.5em"}}><Alert severity={"error"}>{processingError}</Alert></Grid>}
            <Grid item>
                <Button
                    fullWidth
                    variant={"contained"}
                    onClick={() => toggleStatusUpdateDialog(true)}
                    disabled={processingStatusUpdate}
                >
                    Mark Item as Sent {processingStatusUpdate && <Loading/>}
                </Button>
            </Grid>
            <ConfirmDialog
                open={statusUpdateDialog}
                onClose={() => toggleStatusUpdateDialog(false)}
                title={"Confirm Item Delivery"}
                onConfirmed={() => statusUpdate()}
                okBtnText={"CONFIRM"}
            >
                I confirm that item was sent to consumer

            </ConfirmDialog>

        </Grid>
        <Grid item container sx={{padding: '1em 0 1em 0'}} direction={"column"}>
            {processingError &&
                <Grid item sx={{marginBottom: "0.5em"}}><Alert severity={"error"}>{revertError}</Alert></Grid>}
            <Grid item>
                <Button
                    fullWidth
                    variant={"contained"}
                    onClick={() => toggleRevertDialog(true)}
                    disabled={revertingTransaction}
                    color={"error"}
                >
                    Revert Transaction {revertingTransaction && <Loading/>}
                </Button>
            </Grid>
            <ConfirmDialog
                open={showRevertDialog}
                onClose={() => toggleRevertDialog(false)}
                title={"Revert Transaction"}
                onConfirmed={() => revertTransaction()}
                okBtnText={"CONFIRM"}
            >
                Transaction will be marked as reverted. Item will be moved back to stock. Credits will be returned
                to buyer
            </ConfirmDialog>

        </Grid>

    </>
}
export default TransactionActions;
