import React, {FC, Suspense} from "react";
import {observer} from "mobx-react";
import {BrowserRouter, Route, Routes} from "react-router-dom";
import Loading from "../Utils/Loading";
import HistoryGrid from "./routes/history";
import HistoryItem from "./routes/history-item";
import {TStoreStatus, useMarketplaceStore} from "./Store";

const MyListings = React.lazy(() => import("./routes/my"))
const AddListing = React.lazy(() => import("./routes/add"))
const Main = React.lazy(() => import("./routes/main"))
const EditListing = React.lazy(() => import("./routes/edit"))
const Order = React.lazy(() => import("./routes/item"))
const NotFound = React.lazy(() => import("./routes/404"))

const MarketplaceRoutes: FC<{ selfUrl: string }> = observer((props) => {
    const rootStore = useMarketplaceStore();
    return (<>
        {rootStore.status == TStoreStatus.LOADING ? <Loading/> : <BrowserRouter basename={props.selfUrl}>
            <Suspense fallback={<Loading/>}>
                <Routes>
                    <Route path={"/"} element={<Main/>}/>
                    <Route path={"/my"} element={<MyListings/>}/>
                    <Route path={"/my/add"} element={<AddListing/>}/>
                    <Route path={"/my/edit/:id"} element={<EditListing/>}/>
                    <Route path={"/item/:id"} element={<Order type={"all"}/>}/>
                    <Route path={"/my/item/:id"} element={<Order type={"my"}/>}/>
                    <Route path={"/history/:type"} element={<HistoryGrid/>}/>
                    <Route path={"/history/:type/:id"} element={<HistoryItem/>}/>
                    <Route path={"*"} element={<NotFound/>}></Route>
                </Routes>
            </Suspense>
        </BrowserRouter>}
    </>)
})
export default MarketplaceRoutes;