import {MarketplaceStore} from "./index";
import {makeAutoObservable, runInAction} from "mobx";
import {TCategoryType} from "../types";
import {TApiResponseList} from "../../types";
import {CategoryStore} from "../../AdminCategory/Store";

export default class CategoryTypes {
    rootStore: MarketplaceStore;
    types: Array<TCategoryType> = [];
    categories?: Record<string, CategoryStore> = undefined;

    constructor(rootStore: MarketplaceStore) {
        this.rootStore = rootStore;
        makeAutoObservable(this, {
            rootStore: false
        })
    }

    async apiLoadTypes() {
        const response = await this.rootStore.apiRequest('/category/types');
        if (response.ok == true) {
            runInAction(() => {
                this.types = [...(response as TApiResponseList<TCategoryType>).list];
                this.categories = {}
                this.types.map(
                    (type) => {
                        this.categories = Object.assign(this.categories, {
                            [type.type_id]: new CategoryStore({
                                type_id: type.type_id,
                                list: type.list
                            })
                        })
                    })
            })
        }
        return Promise.resolve();
    }

    getCategoriesByType(type_id: string) {
        if (!this.categories)
            throw new Error("Categories are not set");
        return this.categories[type_id];
    }

}