import React, {useEffect, useState} from "react";
import {observer} from "mobx-react";
import MarketplaceLayout from "../Layout";
import {Button, Grid} from "@mui/material";
import {useNavigate, useParams} from "react-router-dom";
import {useMarketplaceStore} from "../Store";
import {LoadingBox} from "../../Utils/Loading";
import ListingItem from "../Layout/Listing/Item";
import TransactionSidebar from "../Layout/Transaction/Sidebar";
import {TTransaction, TTransactionListing} from "../types";
import TransactionSellerSidebar from "../Layout/Transaction/SellerSidebar";


const HistoryItem = observer(() => {
    const navigate = useNavigate();
    const {type = 'orders', id} = useParams();
    const store = useMarketplaceStore();
    const [loading, setLoading] = useState(false);
    const [listing, setListing] = useState<TTransactionListing>();
    const currentUser = store.userStore.user;
    const [transaction, setTransaction] = useState<TTransaction>();

    useEffect(() => {
        setLoading(true);
        store.transactions.apiGetTranactions({type: type, id: id}).then(() => {
            let transaction = store.transactions.list.pop();
            if (transaction) {
                setListing(transaction.listing)
                setTransaction(transaction);
                store.userStore.setTransactionViewed(transaction.transaction_id);
            } else
                navigate('/404');
        }).finally(() => {
            setLoading(false)
        })

    }, [id])

    const handleResponse = (response: any) => {
        if (response.hasOwnProperty('status') && (response.status == 'ok')) {
            if (transaction && response.hasOwnProperty("transaction")) {
                setTransaction(response.transaction as TTransaction);
            }
            return Promise.resolve()
        } else {
            return Promise.reject(response.error)
        }
    }
    const updateComment = (comment: string): Promise<any> => {
        return store.apiRequest("/transaction/comment", {
            transaction_id: transaction?.transaction_id,
            comment: comment
        }).then(handleResponse)
    }
    const confirmSale = (): Promise<any> => {
        return store.apiRequest("/transaction/complete", {
            transaction_id: transaction?.transaction_id
        }).then(handleResponse)
    }
    const confirmDelivery = (): Promise<any> => {
        return store.apiRequest("/transaction/sent", {
            transaction_id: transaction?.transaction_id
        }).then(handleResponse)
    }
    const revertTransaction = (): Promise<any> => {
        return store.apiRequest("/transaction/revert", {
            transaction_id: transaction?.transaction_id
        }).then(handleResponse)
    }

    return <MarketplaceLayout>
        <Grid container direction={"column"}>
            <Grid item>
                <Button onClick={() => navigate(-1)}>&lt;&lt;&lt; back</Button>
            </Grid>
            {loading && <LoadingBox/>}
            {!loading && listing && currentUser && transaction && <ListingItem
                listing={listing}
                sidebar={store.userStore.isSeller(transaction) ?
                    <TransactionSellerSidebar
                        transaction={transaction}
                        onConfirmDelivery={confirmDelivery}
                        onTransactionRevert={revertTransaction}
                    />
                    : <TransactionSidebar
                        transaction={transaction}
                        onCommentUpdate={updateComment}
                        onSaleConfirming={confirmSale}
                    />}
            />}
        </Grid>


    </MarketplaceLayout>
})
export default HistoryItem;
