import {observer} from "mobx-react";
import React, {useEffect, useState} from "react";
import {useListingStore} from "../Store";
import MarketplaceLayout from "../Layout";
import FilterBar from "../Layout/FilterBar";
import {useSearchParams} from "react-router-dom";

const ListingsGrid = React.lazy(() => import("../Layout/ListingsGrid"));

const Main = observer(() => {
    const [isLoading, toggleLoading] = useState(false);
    const listingStore = useListingStore();
    const [searchParams, setSearchParams] = useSearchParams();

    useEffect(() => {
        toggleLoading(true);
        listingStore.apiGetListings({
            all: true,
            filter: listingStore.getFilter(searchParams)
        })
            .finally(() => {
                toggleLoading(false)
            })
    }, [searchParams, listingStore.sort]);

    return <MarketplaceLayout filter={<FilterBar/>}>
        <ListingsGrid records={listingStore.list} isLoading={isLoading}
                      onNeedMoreRecords={() => listingStore.apiGetListings({
                          all: true,
                          skipFirst: listingStore.list.length,
                          filter: listingStore.getFilter(searchParams)
                      })}
                      hasMoreRecords={listingStore.list.length < listingStore.totalRecords}
                      showStatus={false}
                      canDelete={false}
                      canEdit={false}
        />
    </MarketplaceLayout>
})

export default Main;