<?php

namespace Am\Marketplace\Slim;

use Am_Di;
use Bootstrap_Marketplace;
use Bootstrap_SelfService;
use Psr\Container\ContainerInterface;
use Psr\Http\Message\ServerRequestInterface;
use User;

abstract class Controller
{
    protected $container;
    
    function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }
    
    function getDi(): Am_Di
    {
        return $this->container->getDi();
    }
    
    function getModule(): Bootstrap_Marketplace
    {
        return $this->getDi()->modules->loadGet('marketplace');
    }
    
    function getSelfService(): Bootstrap_SelfService
    {
        return $this->getDi()->modules->loadGet("self-service");
    }
    
    function errorResponse($error): Response
    {
        return (new Response\Factory())->createResponse()->withJson([
            'ok' => true,
            'status' => 'error',
            'error' => $error
        ]);
    }
    
    function getBalance(User $user): int
    {
        return $this->getModule()->getBalance($user);
    }
    
    abstract function __invoke(ServerRequestInterface $request, Response $response, array $args): Response;
}