<?php declare(strict_types=1);

namespace Am\Marketplace\Slim\Controller;

use Am\Marketplace\Slim\Controller;
use Am\Marketplace\Slim\Response;
use MarketplaceListing;
use Psr\Http\Message\ServerRequestInterface;

class Purchase extends Controller
{
    
    function __invoke(ServerRequestInterface $request, Response $response, array $args): Response
    {
        $body = $request->getParsedBody();
        $user = $request->getAttribute('user');
        if (!$user) {
            return $this->errorResponse(___('Please login'));
        }
        if (empty($body['listing_id']) || empty($body['qty'])) {
            return $this->errorResponse(___('Missing necessary parameters'));
        }
        
        $qty = max(intval($body['qty']), 1);
        
        $listing = $this->getModule()->listingsTable()->load($body['listing_id']);
        
        try {
            if ($listing->isPurchasePossible($user, $qty)) {
                $total = $qty * $listing->price;
                
                if ($total > $this->getBalance($user)) {
                    $form_id = $this->getModule()->getConfig(___('buy-credits-form'));
                    if (empty($form_id)) {
                        return $this->errorResponse(___('You do not have enough credits to finish purchase'));
                    }
                    /**
                     * @var SavedForm $form ;
                     */
                    $form = $this->getDi()->savedFormTable->load($form_id);
                    
                    
                    $this->getModule()->setPurchaseData($body);
                    
                    $url = $this->getDi()->surl("signup/{$form->code}");
                    
                    return $response->withJson([
                        'ok' => true,
                        'status' => 'redirect',
                        'url' => $url
                    ]);
                    
                }
                
                $transaction = $this
                    ->getModule()
                    ->transactionTable()
                    ->createTransaction()
                    ->setBuyer($user)
                    ->setListing($listing, $qty);
                
                $transaction->insert();
                
                $this->getModule()->newSaleNotification($transaction);
                
                if ($listing->listing_type == MarketplaceListing::TYPE_SERIAL) {
                    $transaction->complete();
                }
                
                return $response->withJson([
                    'ok' => true,
                    'status' => 'ok',
                    'transaction' => $transaction
                ]);
            } else {
                return $response->withJsonError(___("You can't purchase this item!"));
            }
        } catch (Exception $ex) {
            return $response->withJsonError($ex->getMessage());
        }
        
    }
}