<?php declare(strict_types=1);

namespace Am\Marketplace\Slim\Controller;

use Am\Marketplace\Slim\Controller;
use Am\Marketplace\Slim\Response;
use MarketplaceTransaction;
use Psr\Http\Message\ServerRequestInterface;

class TransactionChatPostMessage extends Controller
{
    
    function __invoke(ServerRequestInterface $request, Response $response, array $args): Response
    {
        $params = $request->getParsedBody();
        if (empty($params['transaction_id'])) {
            return $response->withJsonError(___('Empty Transaction'));
        }
        if (empty($params['message'])) {
            return $response->withJsonError(___('Message is Empty'));
        }
        
        /**
         * @var MarketplaceTransaction $transaction
         */
        $transaction = $this->getModule()->transactionTable()->load($params['transaction_id']);
        
        if (!$transaction->isOwner($request->getAttribute('user'))) {
            return $response->withJsonError(___("You can't access this chat"));
        }
        $serverParams = $request->getServerParams();
        $message = $transaction->chat()->postUserMessage($request->getAttribute('user'), $params['message'],
            $serverParams['REMOTE_ADDR'] ?? "", $serverParams['HTTP_USER_AGENT'] ?? "");
        $transaction->recordTransactionUpdate($transaction->getSeller()->pk() == $request->getAttribute('user')->pk() ? $transaction->getBuyer() : $transaction->getSeller());
        
        return (new TransactionChat($this->container))($request, $response, $args);
        
    }
}
