<?php declare(strict_types=1);

namespace Am\Marketplace\Slim\Controller;

use Am\Marketplace\Slim\Controller;
use Am\Marketplace\Slim\Response;
use MarketplaceTransaction;
use Psr\Http\Message\ServerRequestInterface;

class TransactionSent extends Controller
{
    
    function __invoke(ServerRequestInterface $request, Response $response, array $args): Response
    {
        $params = $request->getParsedBody();
        if (empty($params['transaction_id'])) {
            return $this->errorResponse(___('Transaction ID is missing in request'));
        }
        
        $transaction = $this->getModule()->transactionTable()->load($params['transaction_id']);
        
        if (empty($transaction)) {
            $this->errorResponse(___('Unable to find transaction'));
        }
        
        if (($transaction->seller_id != $request->getAttribute('user')->pk()) && ($transaction->status != MarketplaceTransaction::STATUS_PENDING)) {
            $this->errorResponse(___('You are not allowed to change this transaction'));
        }
        
        $transaction->markAsSent();
        
        $transaction->chat()->postSystemMessage(___("Seller has sent this item"));
        
        
        $this->getModule()->transactionStatusChangedNotification($transaction);
        $this->getModule()->transactionStatusChangedNotificationBuyer($transaction);
        return $response->withJson([
            'ok' => true,
            'status' => 'ok',
            'transaction' => $transaction
        ]);
        
    }
}